
#include <iostream>
#include <vector>
#include "project.hpp"
#include "misc.hpp"
#include "annot_proc.hpp"

using std::cerr;
using std::endl;
using std::vector;
using namespace mad;
using namespace mad_ast;

static void usage(char *fname)
{
	char *cp;

	if ((cp = strrchr(fname, '/'))!=NULL) cp++;
	else cp = fname;
	cerr << 
		"**** " PACKAGE " Version " VERSION " (C) 2003 Wei Qin ****\n"
		"usage : " << cp << " [-h] [-q] [-I path]"
		" <file name>\n"
		"  -h : print this message and quit\n"
		"  -q : quiet mode, do not display warnings\n"
		"  -I : search path for MADL files\n"
		"  file name : the main mad file\n";
}


int main(int argc, char *argv[])
{
	int i;
	bool quiet = false;		// donot display warnings
	vector<const char *> ipaths;

	char *filename = NULL;
	for (i=1; i<argc; i++)
	{
		if (strcmp(argv[i], "-h")==0 ||
			strcmp(argv[i], "--help")==0) {
			usage(argv[0]);
		   	return 0;
		}
	   	else if (strcmp(argv[i], "-q")==0 ||
			strcmp(argv[i], "--quiet")==0) {
			quiet = true;
		}
		else if (strcmp(argv[i], "-I")==0 ||
			strcmp(argv[i], "--include")==0) {
			ipaths.push_back(argv[++i]);
		}
		else if (!filename) {
			filename = argv[i];
		}
		else {
			usage(argv[0]);
			return 1;
		}
	}

	if (!filename) {
		usage(argv[0]);
		return 1;
	}

	mad_ast_init(quiet);

	project_manager pm(filename, ipaths, false, false, true, false, false, "");
	if (pm.get_status()) return 0;

	extract_regalloc_info(pm.get_program());
	extract_schedinfo(pm.get_program());

	return 1;
}
