
#ifndef __REGALLOC_ANNOT_HP__
#define __REGALLOC_ANNOT_HP__

#include <vector>
#include "symbol.hpp"

using mad::Symbol;

typedef enum {
  SRC_OPERAND,
  DST_OPERAND
} OperandType;

typedef enum {
  OPTIONS_CONSTR,
  ASSIGN_CONSTR,
  BANK_CONSTR,
  VAR_CONSTR,
} ConstraintType;

typedef struct _variable {
  Symbol name;
  OperandType type;
} Variable;

typedef std::vector<Variable *> VariableList;

typedef struct _register {
  Symbol name;
} Register;

typedef std::vector<Register *> RegisterList;

typedef struct register_bank {
  Symbol name;
  RegisterList *regs;
} RegisterBank;

typedef std::vector<RegisterBank *> RegisterBankList;

typedef struct overlap_regs {
  Register *reg1;
  Register *reg2;
} OverlapRegs;

typedef std::vector<OverlapRegs *> OverlapConstraints;

typedef struct regoptions_constr {
  std::vector<Variable *> *vars;
  std::vector<RegisterList *> *options;
} RegoptionsConstraint;

typedef struct regassign_constr {
  Variable *var;
  Register *reg;
} RegassignConstraint;

typedef struct bank_constr {
  Variable *var;
  RegisterBank *bank;
}BankConstraint;

typedef struct var_constr {
  Variable *var1;
  Variable *var2;
} VarConstraint;

typedef struct constraint {
  ConstraintType type;
  union {
    RegoptionsConstraint *aconstr;
    BankConstraint *bconstr;
    VarConstraint *vconstr;
    RegassignConstraint *rconstr;
  } constr;
} Constraint;

typedef std::vector<Constraint *> ConstraintList;

typedef struct oper {
  Symbol name;
  VariableList *operands;
  ConstraintList *constraints;
} Oper;

typedef std::vector<Oper *> OperList;

typedef struct regallocspec {
  RegisterList *reglist;
  RegisterBankList *regbanklist;
  OverlapConstraints *overlaplist;
  OperList *operlist;
} RegallocSpec;

#endif
