%{

#include <vector>

#include "decode.h"

#define YYERROR_VERBOSE

extern "C" {
int yylex();
}

void yyerror(char *);
extern void push_entry(char *, char *, char *, double);

extern int mylineno;
extern std::vector<DecodeEntry> entries;

%}

%union {
	double fltval;
	char *strval;
}

%token IDEF LBRAC RBRAC COMMA

%token <strval> IDENT  "identifier"
%token <strval> HCONST "heximal constant"
%token <strval> DCONST "decimal constant"
%token <fltval> FCONST "floating point constant"

%start prog

%%

prog: stmt {}
	| prog stmt {};

stmt:	IDEF LBRAC IDENT COMMA HCONST COMMA HCONST COMMA FCONST RBRAC {
		push_entry($3,$5,$7,$9);
	}
	| 	IDEF LBRAC IDENT COMMA HCONST COMMA HCONST COMMA DCONST RBRAC {
		push_entry($3,$5,$7,strtod($9,NULL));
	};

%%

void yyerror (char *s)
{
	std::cerr << s << "near line " << mylineno+1 << std::endl;
}

void push_entry(char *id, char *mask, char *sign, double prob)
{
	DecodeEntry tmp(mask,sign,id,prob);
	entries.push_back(tmp);
	free(id),free(mask),free(sign); /* free the memory */
}

