/*
	GRAPH FORMAT DEFINITIONS FILE
*/

/*
        Title: Graph Definitions File.
        file: graph.h
        does: 
	        Graph definition and input
	        Macros for access to graph
	        cheat Info from Graph input

        Source: Joseph Culberson's Coloring Page
                http://web.cs.ualberta.ca/~joe/Coloring/index.html
        Author: Joseph Culberson
        email: joe@cs.ualberta.ca

	Copyright (c) 1997 Joseph Culberson. All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
   	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
   	notice, this list of conditions and the following disclaimer in the
   	documentation and/or other materials provided with the distribution.
	3. All advertising materials mentioning features or use of this 
	software must display the following acknowledgement:
     	This product includes software developed by J. Culberson at the
     	University of Alberta, Edmonton.
	4. Neither the name of the University nor the names of its contributors
   	may be used to endorse or promote products derived from this software
   	without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
	PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	THIS SOFTWARE IS SUPPLIED WITHOUT ANY SUPPORT SERVICES. 


*/

/*
	Graph definition and input
	Macros for access to graph
	Cheat Info from Graph input
*/
#ifndef GRAPHDEFS
#define GRAPHDEFS

/*
	!!!WARNING!!!!
	MAXVERTEX must be divisible by 8
*/
#define MAXVERTEX  2000
#define SHIFT 3
#define MASK 7
#define ROWSIZE ((MAXVERTEX >> SHIFT) +1 )
#define GRAPHSIZE  (MAXVERTEX * ROWSIZE)

#define setedge(i,j)  graph[((i)*ROWSIZE) + ((j) >> SHIFT)] |= (1 << ((j) & MASK))

typedef int vertextype;
typedef unsigned char adjacencytype;
typedef unsigned short colortype;

typedef struct clrinfo {                                              
	colortype numcolors; /* number of colors used */
	int total;  /* sum of colors (weight)  used */    
} clrinfotype;

typedef struct vrtxandclr {
	vertextype vertex;
	colortype color;
} vrtxandclrtype;

typedef struct popmember {
	clrinfotype clrdata;
	vrtxandclrtype vc[MAXVERTEX];
} popmembertype;

extern int greedy_color(adjacencytype *graph, vertextype order,
	popmembertype *m);

#endif
