" Vim syntax file
" Language:	MADL (Mescal Architecture Description Language)
" Maintainer:	Wei Qin <wqin@ee.princeton.edu>
" Last Change:	$Date: 2003/10/02 03:05:32 $

" put this into to your ~/.vimrc
"
"    augroup syntax
"    au! BufNewFile,BufReadPost *.mad
"    au  BufNewFile,BufReadPost *.mad  so ~/vim/mad.vim
"    augroup END
"
" ... and save this file as ~/vim/mad.vim ;-)
"
"-------------------------------------------"
" I TESTED THIS ONLY ON A CTERM"
"-------------------------------------------"

" Remove any old syntax stuff hanging around
syn clear

" keyword definitions

" level 1 keyword
syn keyword madKeyword1 INCLUDE
syn keyword madKeyword1 IMPORT
syn keyword madKeyword1 DEFINE
syn keyword madKeyword1 MANAGER 
syn keyword madKeyword1 MACHINE
syn keyword madKeyword1 OPERATION
syn keyword madKeyword1 FUNCTION
syn keyword madKeyword1 USING

" level 2 keyword
syn keyword madKeyword2 VAR
syn keyword madKeyword2 SYNTAX
syn keyword madKeyword2 CODING
syn keyword madKeyword2 TRANS
syn keyword madKeyword2 EVAL
syn keyword madKeyword2 EDGE
syn keyword madKeyword2 BUFFER
syn keyword madKeyword2 STATE
syn keyword madKeyword2 INITIAL
syn keyword madKeyword2 CLASS
syn keyword madKeyword2 INSTANCE


" Types
syn keyword madType uint
syn keyword madType int
syn keyword madType void
syn keyword madType string
syn keyword madType float
syn keyword madType double


" Operators
"syn keyword madOperator         "<<"
"syn keyword madOperator         "+"
"syn keyword madOperator         "-"
"syn keyword madOperator         "*"
"syn keyword madOperator         "/"
"syn keyword madOperator         "="
"syn keyword madOperator         "<="
"syn keyword madOperator         ">="
"syn keyword madOperator         "<"
"syn keyword madOperator         ">"
"syn keyword madOperator         ".."
"syn keyword madOperator         "."
"syn keyword madOperator         "->"
"syn keyword madOperator         "|"
"syn keyword madOperator         "&"
"syn keyword madOperator         "!"
"syn keyword madOperator         "::"

" Constants
"integer number, or floating point number without a dot and with "f".
"floating point number, without dot, with exponent
syn match   madFloat   "[-+]\=\d\+[eE][-+]\=\d\+[fF]\=\>"
"floating point number, with dot, optional exponent
syn match   madFloat   "[-+]\=\d*\.\d\+\([eE][-+]\=\d\+\)\=[fF]\=\>"
"floating point number, starting with a dot, optional exponent
syn match   madFloat   "[-+]\=\d\+\.\d*\(e[-+]\=\d\+\)\=[fF]\=\>"

syn match  madNumber "\<\d\+\>"
syn match  madNumber "\<0[xX]\x\+\>"
syn match  madNumber "\<0\o\+\>"
syn match  madNumber "\<0[bB][01]\+\>"


"Comment
syn match   madComment      "#.*"
syn region  madComment      start="##" end="##"

"Annotation
syn match   madAnnotation   "\$.*"
syn region  madAnnotation   start="\$\$" end="\$\$"

" Catch mismatched parentheses
syn region madBrack transparent start="\[" end="\]" contains=ALLBUT,madBrackError
syn region madParen transparent start="(" end=")" contains=ALLBUT,madParenError
syn region madUnion transparent start="{" end="}" contains=ALLBUT,madBraceError
syn match madParenError     ")"
syn match madBrackError     "]"
syn match madBraceError     "}"

syn region  madString start=+"+  skip=+\\\\\|\\"+ end=+"+

syn sync lines=10

if !exists("did_mad_syntax_inits")
  let did_mad_syntax_inits = 1
  " The default methods for highlighting.  Can be overridden later
  hi link madKeyword1		Type
  hi link madKeyword2		Statement
  hi link madNumber 		Number
  hi link madPattern		Number
  hi link madFloat		Number
  hi link madOperator		Statement
  hi link madComment		Comment
  hi link madType	 	Type
  hi link madParenError		Error
  hi link madBracketError	Error
  hi link madBraceError		Error
  hi link madString		String
  hi link madAnnotation		MoreMsg
endif

let b:current_syntax = "mad"

" vim: ts=8
