
#include <cstdio>
#include <csignal>
#include <cstdlib>
#include "simulator.hpp"
#include "loader.h"
#include "syscall.h"

static void usage(char *fname)
{
	char *cp;
	if ((cp = rindex(fname, '/'))!=NULL) cp++;
	else cp = fname;
	fprintf(stderr, 
		"**** " PACKAGE " Version " VERSION " (C) 2002,2003 Wei Qin ****\n"
		"usage : %s [-d] [-v] [-f fpe-path] [-h] <file name> <args...>\n"
		//"  -d : debuging mode\n"
		"  -v : verbose\n"
		"  -m : maximum number of cycles to simulate\n"
		"  -f : FPE binary file path\n"
		"  -h : print this message and quit\n"
		"  file name : the ARM ELF32 program to simulate\n"
		"  args : arguments to the program\n", cp);
}

arm_simulator *sima;

void sigint_handler(int signum)
{
	if (sima->is_running()) {
		sima->stop();
	}
	else exit(0);
}

int main(int argc, char *argv[], char *envp[])
{
	int i;
	char *prog_name = NULL;
	char *fpe_name = NULL;
	bool verbose = false;
	//bool debuging = false;
	uint64_t max_cnum = (uint64_t)-1;
	void (*prev_sigint_handler)(int);

	for(i = 1; i < argc; i++)
	{
		//if(strcmp(argv[i], "-d") == 0) debuging = true; else
		if(strcmp(argv[i], "-v") == 0) verbose = true; else
		if(strcmp(argv[i], "-f") == 0) fpe_name = argv[++i]; else
		if(strcmp(argv[i], "-m") == 0) max_cnum = ato_uint64(argv[++i]); else
		if(strcmp(argv[i], "-h") == 0) {usage(argv[0]); return 0;} else
		if(!prog_name)
		{
			prog_name = argv[i];
			break;
		}	
	}
	
	/* emulator instance */
	if(prog_name)
	{
		sima = new arm_simulator(verbose);
		prev_sigint_handler = signal(SIGINT, sigint_handler); 

	//	sima->load_fpe(fpe_name);
		sima->load_program(prog_name, argc - i, argv + i, envp);
		sima->run(max_cnum);
		if(prev_sigint_handler != SIG_ERR) signal(SIGINT, SIG_DFL);
	}
	else usage(argv[0]);

	return 0;
}

