
#include <iostream>
#include <sstream>
#include "interface.hpp"
#include "syscall.h"

using std::ostream;
using std::stringstream;

#define UFLD ((inst>>23)&1)
#define PFLD ((inst>>24)&1)
#define WFLD ((inst>>21)&1)
#define LFLD ((inst>>20)&1)

extern STR_T cond_names[16];
extern STR_T reg_names[16];

/* external functions defined */
void sys_call(arm_simulator *sm, const BIT_T(WORD_T,32)& iw){
	do_syscall(sm, iw.val());
}

/* get the right most bit from bit ind, for ldm/stm simulation use */
void rmb(BIT_T(WORD_T,4)& val, const BIT_T(WORD_T,32)& ival,
				 const BIT_T(WORD_T,4)& ind)
{
	unsigned v = ival.val();
	for (int i=ind.val(); i<16; i++) {
		if (v&(1<<i)) {
			val=i;
			return;
		}
	}
	/* if ival==0, then the result is not set. */ 
}

static void disasm_ldstm(arm_inst_t inst, arm_addr_t addr, ostream& strm)
{
	UInt32 i, first = 1;
	
	strm << (LFLD?"ldm":"stm") << cond_names[COND]
		<< (UFLD?"i":"d") << (PFLD?"b":"a") << " "
		<< reg_names[RNFLD] << (WFLD?"!":"") << ", {";

	for (i = 0; i<16; i++) {
		if ((1<<i) & inst) {
			strm << (first?"":", ") << reg_names[i];
			first = 0;
		}
	}
	strm << "}";

	if (BITn(inst,22)) strm << "^";
}

void ldstm_syntax(STR_T &val, const BIT_T(WORD_T, 32)& iw)
{
	stringstream sstr;
	disasm_ldstm(iw.val(), 0, sstr);
	val = sstr.str();
}

/* Unknown opcode */
OPER_ID_T *_impl_default(unsigned iw) {
	static OPER_ID_T hehe[] = {4,32,0,0};
	//assert(0);
	return hehe;
}

