
#ifndef __FLOAT_TYPE_H__
#define __FLOAT_TYPE_H__

#include <sstream>
#include <string>
#include "bit_type.hpp"

#define FLT_T _float_t<float>
#define DBL_T _float_t<double>

#define FLT_CTR(_v) _float_t<float>(_v)
#define DBL_CTR(_v) _float_t<double>(_v)

#define BIT2FLT(_t1,_i1,_v) FLT_T((float)(VAL_OF(_v)))
#define BIT2DBL(_t1,_i1,_v) DBL_T((double)(VAL_OF(_v)))

#define DBL2FLT(_v) FLT_T((float)(VAL_OF(_v)))
#define FLT2DBL(_v) DBL_T((double)(VAL_OF(_v)))

#define DBL2BIT(_t1,_i1,_v) BIT_CTR(_t1,_i1,(_t1)(VAL_OF(_v)))
#define FLT2BIT(_t1,_i1,_v) BIT_CTR(_t1,_i1,(_t1)(VAL_OF(_v)))

#define VAL_OF(_a) (_a).val()

/* short bit class template, for i<=32 */
template <class T> class _float_t {

	public:

		/* Constructor 1 */
		_float_t() : value(0.0) {}

		/* Constructor 2 */
		_float_t(const T& v) {
			value =  v;
		}

		/* Assignment operator */
		_float_t<T>& operator = (const T& v) {
			value = v;
			return *this;
		}

		/* Get the value */
		T val() const {return value;}

		std::string sci() {
			std::stringstream sstr;
			sstr << std::scientific << value;
			return sstr.str();
		}

		std::string fix() {
			std::stringstream sstr;
			sstr << std::fixed << value;
			return sstr.str();
		}

	private:
		T value;
};

template <class T> 
inline const _float_t<T> operator + (const _float_t<T>& x,
			   						const _float_t<T>& y) {
	return _float_t<T> (x.val() + y.val());
}

template <class T> 
inline const _float_t<T> operator - (const _float_t<T>& x,
			   						const _float_t<T>& y) {
	_float_t<T> __result(x.val() - y.val());
	return __result;
}

template <class T> 
inline const _float_t<T> operator * (const _float_t<T>& x,
			   						const _float_t<T>& y) {
	_float_t<T> __result(x.val() * y.val());
	return __result;
}

template <class T> 
inline const _float_t<T> operator / (const _float_t<T>& x,
			   						const _float_t<T>& y) {
	_float_t<T> __result(x.val() / y.val());
	return __result;
}

template <class T>
inline const _float_t<T> operator - (const _float_t<T>&x)
{
	_float_t<T> __result(-x.val());
	return __result;
}

template <class T> 
inline const _bit_t<unsigned, 1> operator == (const _float_t<T>& x,
											const _float_t<T>& y) {
	return x.val() == y.val();
}

template <class T> 
inline const _bit_t<unsigned, 1> operator != (const _float_t<T>& x,
											const _float_t<T>& y) {
	return x.val() != y.val();
}

template <class T> 
inline const _bit_t<unsigned, 1> operator <  (const _float_t<T>& x,
											const _float_t<T>& y) {
	return x.val() < y.val();
}

template <class T>
inline const _bit_t<unsigned, 1> operator >  (const _float_t<T>& x,
											const _float_t<T>& y) {
	return x.val() > y.val();
}

template <class T> 
inline const _bit_t<unsigned, 1> operator <=  (const _float_t<T>& x,
											const _float_t<T>& y) {
	return x.val() <= y.val();
}

template <class T>
inline const _bit_t<unsigned, 1> operator >=  (const _float_t<T>& x,
											const _float_t<T>& y) {
	return x.val() >= y.val();
}

#endif
