
#ifndef __SIMPLE_SCHED_H__
#define __SIMPLE_SCHED_H__

#include <list>
#include "osm.hpp"

#ifdef DEBUG
#include <iostream>
#endif

using std::list;

/** The scheduler of OSMs.
 *  @param S The simulator class.
 */
template <class S>
class scheduler {

	public:
		/** Constructor.
		 *  @param sim The simulator object.
		 */
		scheduler(S *sim) : workList(), sim(sim) {}

		/** Destructor. */
		~scheduler() {}

		/** A control step. */
		void step() {

			list<_base_machine *>::iterator it;

			/** Activate all OSMs in order. */
			for (it=workList.begin(); it!=workList.end();) {
		
#ifdef DEBUG
				int oldstate = (*it)->get_state();
#endif
				bool ret = (*it)->activate();
#ifdef DEBUG
				std::cerr << (*it)->get_dynid() << " " << oldstate << "->";
				std::cerr << (*it)->get_state() << " " << ret << std::endl;
#endif

				/* Test if the OSM is back to the initial state. */
				if (ret) { /* no */
					it++;
				}
				else { /* yes */
					/* send retired ones to the sink. */
					sim->sink_machine(*it, false);
					it = workList.erase(it);
				}
			}

			/* get new OSMs from the source. */
			bool change = true;
			while (change) {
				change = false;
				_base_machine *src = sim->source_machine(true);
				if (src->activate()) {
					workList.push_back(src);
					sim->source_machine(false);
				}
			}
		}

		/* return true if the work list is empty */
		bool empty() const {return workList.empty();}

		/* flush objects with larger dynids than this. */
		void flush(const uint64_t& id) {
			
			list<_base_machine *>::iterator it;
			for (it=workList.begin(); it!=workList.end();) {
				if ((*it)->get_dynid()>id && (*it)->get_state()>1) {
					sim->sink_machine(*it, true);
					it = workList.erase(it);
				}
				else it++;
			}
		}

	private:

		std::list<_base_machine *> workList;
		S *sim;
};

#endif
