
#include <cstdio>
#include <csignal>
#include <cstdlib>
#include "simulator.hpp"
#include "loader.h"
#include "syscall.h"

#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#endif

uint64_t _count;

static void usage(char *fname)
{
	char *cp;
	if ((cp = rindex(fname, '/'))!=NULL) cp++;
	else cp = fname;
	fprintf(stderr, 
		"**** " PACKAGE " Version " VERSION " (C) 2002,2003 Wei Qin ****\n"
		"usage : %s [-d] [-v] [-f fpe-path] [-h] <file name> <args...>\n"
		//"  -d : debuging mode\n"
		"  -v : verbose\n"
		"  -m : maximum number of cycles to simulate\n"
		"  -f : FPE binary file path\n"
		"  -h : print this message and quit\n"
		"  file name : the ARM ELF32 program to simulate\n"
		"  args : arguments to the program\n", cp);
}

arm_simulator *sima;

void sigint_handler(int signum)
{
	if (sima->is_running()) {
		sima->stop();
	}
	else exit(0);
}

int main(int argc, char *argv[], char *envp[])
{
	int i;
	char *prog_name = NULL;
	char *fpe_name = NULL;
	bool verbose = false;
	//bool debuging = false;
	uint64_t max_cnum = (uint64_t)-1;
	void (*prev_sigint_handler)(int);

	for(i = 1; i < argc; i++)
	{
		//if(strcmp(argv[i], "-d") == 0) debuging = true; else
		if(strcmp(argv[i], "-v") == 0) verbose = true; else
		if(strcmp(argv[i], "-f") == 0) fpe_name = argv[++i]; else
		if(strcmp(argv[i], "-m") == 0) max_cnum = ato_uint64(argv[++i]); else
		if(strcmp(argv[i], "-h") == 0) {usage(argv[0]); return 0;} else
		if(!prog_name)
		{
			prog_name = argv[i];
			break;
		}	
	}
	
	/* emulator instance */
	if(prog_name)
	{
		sima = new arm_simulator(verbose);
		prev_sigint_handler = signal(SIGINT, sigint_handler); 

	//	sima->load_fpe(fpe_name);
		sima->load_program(prog_name, argc - i, argv + i, envp);

#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H   
		struct timeval begin_u, end_u, begin_s, end_s;			
		struct rusage usg;										
		float user_time, sys_time;								
		getrusage(RUSAGE_SELF, &usg);							 
		begin_u = usg.ru_utime;								   
		begin_s = usg.ru_stime;								   
#endif

		sima->run(max_cnum);
		if(prev_sigint_handler != SIG_ERR) signal(SIGINT, SIG_DFL);

		sima->dump_stats(stderr);

		fprintf(stderr, "%lld %d\n", _count, sizeof(_edge_));
#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H					
		getrusage(RUSAGE_SELF, &usg);							 
		end_u = usg.ru_utime;									 
		end_s = usg.ru_stime;									 
		user_time = (end_u.tv_sec+end_u.tv_usec/1000000.0)-	   
			(begin_u.tv_sec+begin_u.tv_usec/1000000.0);		   
		sys_time = (end_s.tv_sec+end_s.tv_usec/1000000.0)-		
			(begin_s.tv_sec+begin_s.tv_usec/1000000.0);		   
		fprintf(stderr,										   
			"Total user time  : %.3f sec.\n"					  
			"Total system time: %.3f sec.\n"					  
			"Simulation speed : %.3e cyc./sec.\n",				
			user_time, sys_time, sima->get_cycle_count()/(user_time+sys_time));				
#endif

	}
	else usage(argv[0]);

	return 0;
}

