
#define PADDEDHEX "0x" << std::hex << std::setw(8) << std::setfill('0')

#include "more_managers.hpp"

void fetch_manager::update_on_clock()
{
	/* a new OSM comes in */
	if (trigger) {
		current_pc = mRF->get_pc().val();
		mRF->set_pc(current_pc+4);
		trigger = false;
	}
	/* need to change the fetch address */
	else if (mRF->pc_modified()) {
		fetch_status = token->is_free()?0:1;
		current_pc = mRF->get_pc().val();
		mRF->set_pc(current_pc+4);
	}

	if (fetch_status==1) {
		if (itlb->read(current_pc, 4)) {
			fetch_status = 2;
		}
	}
	if (fetch_status==2) {
		if (icache->read(current_pc, 4)) {
			fetch_status = 0;
		}
	}
}

bool fetch_manager::token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj)
{
	if (fetch_status==0) {

		unsigned iw = mem->read_word_fast(current_pc);
		value = iw;

#ifdef DEBUG
	fprintf(stderr, "0x%08x : 0x%08x\n", current_pc, iw);
#endif
		return true;
	}
	return false;
}

void new_pc::update_on_clock()
{
	if (trigger) {
		/* see if an older operation has updated pc */
		if (!mRF->pc_modified()) { 
			mRF->set_pc(npc, dynid);
		}
		trigger = false;
	}
}

void mem_ctrl_port::update_on_clock() 
{
	if (trigger) {

		assert(status==0);

		/* if the pc is modified, do nothing */
		if (mRF->pc_modified() && mRF->get_dynid()<dynid) {
			trigger = false;
			return;
		}

		status = 1;
		addr = mMemAddr->get_value().val();
		mergeable = count > 0;
		trigger = false;
	}
	else if (count>0) {

		if (status==0 && --count>0) {
			status = 1;
			addr += 4;
		}
	}

	if (status==1) {
		if (dtlb->read(addr, size))
			status = 2;
	}

	if (status==2) {
		if (isRead && dcache->read(addr, size)) {
			if (size==4) {
				mMemRead->set_value(mem->read_word(addr));
#if 0
				if (mMemRead->get_value().val()!=emul->mem.read_word(addr))
					std::cerr << "read wong value " << std::hex << addr
						<< " " << mMemRead->get_value().val() << " "
						<< emul->mem.read_word(addr) << std::endl;
			std::cerr << "read value " << std::hex << addr
				<< " " << mMemRead->get_value().val() << std::endl;
#endif
			}
			else if (size==1) {
				mMemRead->set_value(mem->read_byte(addr));
#if 0
				if (mMemRead->get_value().val()!=emul->mem.read_byte(addr))
					std::cerr << "read wong value " << std::hex << addr
						<< " " << mMemRead->get_value().val() << " "
						<< emul->mem.read_byte(addr) << std::endl;
#endif
			}
			else {
				mMemRead->set_value(mem->read_half_word(addr));
			}
			status = 0;
#if 0
			std::cerr << "read once " << std::hex << addr << " "
				   << mMemRead->get_value() << " "	;
#endif
		}
		if (!isRead && dcache->write(addr, size, mergeable)) {
			if (size==4) {
				mem->write_word(addr, mMemWrite->get_value().val());
#if 0
				if (mMemWrite->get_value().val()!=emul->mem.read_word(addr))
					std::cerr << "write wong value " << std::hex << addr
						<< " " << mMemWrite->get_value().val() << " "
						<< emul->mem.read_word(addr) << std::endl;
#endif
					
			}
			else if (size==1) {
				mem->write_byte(addr, mMemWrite->get_value().val());
			}
			else {
				mem->write_half_word(addr, mMemWrite->get_value().val());
			}
			status = 0;
#if 0
			std::cerr << "write once " << std::hex << addr << " "
				   << mMemWrite->get_value() << " "	;
#endif
		}
	}
}

