
#ifndef __FLOAT_TYPE_H__
#define __FLOAT_TYPE_H__

#include <sstream>
#include <string>
#include "bit_type.hpp"

#define _FLT_T _float_t_<float>
#define _DBL_T _float_t_<double>

#define _FLT_CTR(_v) _float_t_<float>(_v)
#define _DBL_CTR(_v) _float_t_<double>(_v)

#define _BIT2FLT(_t1,_i1,_v) _FLT_T((float)(_VAL((_v))))
#define _BIT2DBL(_t1,_i1,_v) _DBL_T((double)(_VAL((_v))))

#define _DBL2FLT(_v) _FLT_T((float)(_VAL((_v))))
#define _FLT2DBL(_v) _DBL_T((double)(_VAL((_v))))

#define _DBL2BIT(_t1,_i1,_v) _BIT_CTR(_t1,_i1,(_t1)(_VAL((_v))))
#define _FLT2BIT(_t1,_i1,_v) _BIT_CTR(_t1,_i1,(_t1)(_VAL((_v))))

#define _VAL(_a) (_a).val()

#define _FLT2I(_i,_v) _int_cast_<_i,_FLT_T >()(_v)
#define _FLT2U(_i,_v) _uint_cast_<_i,_FLT_T >()(_v)
#define _DBL2I(_i,_v) _int_cast_<_i,_DBL_T >()(_v)
#define _DBL2U(_i,_v) _uint_cast_<_i,_DBL_T >()(_v)

#define _U2FLT(_i,_v) _FLT_T((float)(_VAL((_v))))
#define _U2DBL(_i,_v) _DBL_T((double)(_VAL((_v))))
#define _I2FLT(_i,_v) _FLT_T((float)(_VAL((_v))))
#define _I2DBL(_i,_v) _DBL_T((double)(_VAL((_v))))



/* short bit class template, for i<=32 */
template <class T> class _float_t_ {

	public:

		/* Constructor 1 */
		_float_t_() : _value_(0.0) {}

		/* Constructor 2 */
		_float_t_(const T& v) {
			_value_ =  v;
		}

		/* Assignment operator */
		_float_t_<T>& operator = (const T& v) {
			_value_ = v;
			return *this;
		}

		/* Get the _value_ */
		T val() const {return _value_;}

		std::string sci() {
			std::stringstream sstr;
			sstr << std::scientific << _value_;
			return sstr.str();
		}

		std::string fix() {
			std::stringstream sstr;
			sstr << std::fixed << _value_;
			return sstr.str();
		}

	private:
		T _value_;
};

template <class T> 
inline const _float_t_<T> operator + (const _float_t_<T>& x,
			   						const _float_t_<T>& y) {
	_float_t_<T> _result_(x.val() + y.val());
	return _result_;
}

template <class T> 
inline const _float_t_<T> operator - (const _float_t_<T>& x,
			   						const _float_t_<T>& y) {
	_float_t_<T> _result_(x.val() - y.val());
	return _result_;
}

template <class T> 
inline const _float_t_<T> operator * (const _float_t_<T>& x,
			   						const _float_t_<T>& y) {
	_float_t_<T> _result_(x.val() * y.val());
	return _result_;
}

template <class T> 
inline const _float_t_<T> operator / (const _float_t_<T>& x,
			   						const _float_t_<T>& y) {
	_float_t_<T> _result_(x.val() / y.val());
	return _result_;
}

template <class T>
inline const _float_t_<T> operator - (const _float_t_<T>&x)
{
	_float_t_<T> _result_(-x.val());
	return _result_;
}

template <class T> 
inline const _bit_t_<unsigned, 1> operator == (const _float_t_<T>& x,
											const _float_t_<T>& y) {
	return x.val() == y.val();
}

template <class T> 
inline const _bit_t_<unsigned, 1> operator != (const _float_t_<T>& x,
											const _float_t_<T>& y) {
	return x.val() != y.val();
}

template <class T> 
inline const _bit_t_<unsigned, 1> operator <  (const _float_t_<T>& x,
											const _float_t_<T>& y) {
	return x.val() < y.val();
}

template <class T>
inline const _bit_t_<unsigned, 1> operator >  (const _float_t_<T>& x,
											const _float_t_<T>& y) {
	return x.val() > y.val();
}

template <class T> 
inline const _bit_t_<unsigned, 1> operator <=  (const _float_t_<T>& x,
											const _float_t_<T>& y) {
	return x.val() <= y.val();
}

template <class T>
inline const _bit_t_<unsigned, 1> operator >=  (const _float_t_<T>& x,
											const _float_t_<T>& y) {
	return x.val() >= y.val();
}

#endif
