
#ifndef __LIB_OSM_H__
#define __LIB_OSM_H__

/* user of this file must provide macro
 * _MACH_ID_T and _OPER_ID_T
 */
#ifndef _MACH_ID_T
#error "_MACH_ID_T undefined!"
#else
#ifndef _OPER_ID_T
#error "_OPER_ID_T undefined!"
#else
#ifndef _SIMULATOR
#error "_SIMULATOR undefined!"
#else

#include "lib_types.hpp"
#include "managers.hpp"
#include "osm.hpp"

/* Dummy left shift to adjust the type of the two branches so that
 * they are identical. */
#define _TOF(_a,_b,_c) _a?(_b<<0):(_c<<0)

#define _FUN_REF(_fname, _args...) _fname(_args)
#define _E_FUN_REF(_fname, _args...) _fname(_args)
#define _DIRTY_FUN_REF(_fname, _args...) _fname(_args)
#define _DIRTY_E_FUN_REF(_fname, _args...) \
			_fname(_machine_->__get_simulator(),_args)

#define _DIRTY_E_FUN_DEC(_fname, _args...) \
			extern void _fname(_SIMULATOR *, _args)
#define _E_FUN_DEC(_fname, _args...) \
			extern void _fname(_args)

#define _DIRTY_FUN_DEC(_fname, _args...) \
			void _fname(_SIMULATOR *_sim_, _args)
#define _FUN_DEC(_fname, _args...) \
			void _fname(_args)

#if 0 /* no longer used */
#define _VAR_REF(_a) _a
#define _ARG_REF(_a) _a
#define _DEF_REF(_a) _a
#define _MAN_REF(_a) _a
#endif
#define _MVR(_a) _machine_->_a
#define _BUF(_a) _machine_->_a

#ifndef _BASE_OPERATION
#define _BASE_OPERATION _base_operation_
#endif

#ifndef _BASE_MACHINE
#define _BASE_MACHINE _base_machine_
#endif

#define _OPER_CLASS(_a) OC_ ## _a
#define _MACH_CLASS(_a) MC_ ## _a
#define _MANG_CLASS(_a) _a

#define _TEST_RELEASE(_a) _BUF(_a)->test_release(_machine_)

#define _RELEASE(_a) _BUF(_a)->release(_machine_), _BUF(_a)=NULL

#define _TEST_ALLOCATE(_a) \
		_machine_->__get_simulator()->_a\
			->token_free_for_alloc(_machine_)

#define _TEST_ALLOCATE_ID(_a,_b) \
		_machine_->__get_simulator()->_a\
			->token_free_for_alloc(_b,_machine_)

#define _ALLOCATE(_a) \
		_machine_->__get_simulator()->_a\
			->allocate_token(_machine_)

#define _ALLOCATE_ID(_a,_b) \
		_machine_->__get_simulator()->_a\
			->allocate_token(_b,_machine_)

#define _DISCARD(_a) _BUF(_a)->discard(_machine_), _BUF(_a)=NULL

#define _TEST_INQUIRE(_a) \
		_machine_->__get_simulator()->_a\
			->token_free_for_inquire(_machine_)

#define _TEST_INQUIRE_ID(_a,_b) \
		_machine_->__get_simulator()->_a\
			->token_free_for_inquire(_b,_machine_)

#define _READ_TOKEN(_a,_b) \
		_a = _machine_->__get_simulator()->_b\
			->read_token(_machine_)

#define _READ_TOKEN_ID(_a,_b,_c) \
		_a = _machine_->__get_simulator()->_b\
			->read_token(_c,_machine_)

#define _READ(_a, _b) _a = _BUF(_b)->read(_machine_)

#define _WRITE_TOKEN(_a,_b) \
		_machine_->__get_simulator()->_a\
			->write_token(_b,_machine_)

#define _WRITE_TOKEN_ID(_a,_b,_c) \
		_machine_->__get_simulator()->_a\
			->write_token(_b,_c,_machine_)

#define _WRITE(_a,_b) _BUF(_a)->write(_b,_machine_)

#define _RUN_EDGE_EVAL(_id) _machine_->__get_edge(_id)->eval()

#ifdef _GEN_DISASSEMBLER
#define _STRING_EVAL(_a) _a 
#else
#define _STRING_EVAL(_a)
#endif

#define _GET_OPERATION(_a) _machine_->__get_simulator()->allocate_operation(_a);

#define _DECODE(_a,_b) _a(_VAL(_b))

#define _ACT_OPERATION(_a, _b, _c) \
		_machine_->__get_simulator()->activate_operation(_a, _b, _c);

#define _COMPARE(_a,_b,_c) _VAL((_a _b _c))

#define _RETIRE_BUFFER(_a) /*assert(a==NULL)*/

#define _FLUSH_BUFFER(_a)  if (_a!=NULL) {\
		_a->discard(this); \
		_a = NULL; \
	}

#endif
#endif
#endif
#endif
