
#include "simulator.hpp"

#include "opers.hpp"
#include "machines.hpp"
#include <vector>
#include <iomanip>
#include <iostream>

#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include "config.h"

using std::vector;
using std::list;

arm_simulator::arm_simulator(bool verbose) : verbose(verbose), mem()
{
	/* storage for operation/machine */
	machine_pool = new obj_pool<_base_machine_, _MACH_ID_T, arm_simulator,
								arm_simulator::mach_allocator>(_mach_total);
	operation_pool = new obj_pool<_base_operation_, _OPER_ID_T, arm_simulator,
								arm_simulator::oper_allocator>(_oper_total);
	
	/* reset every thing */
	reset_stats();
}

arm_simulator::~arm_simulator()
{
	delete machine_pool;
	delete operation_pool;
}


void arm_simulator::reset_stats()
{
}

void arm_simulator::dump_stats(FILE *fp)
{
#if 0
	fprintf(fp, "Total allocated OSMs : %lld\n", alloc_count);
	fprintf(fp, "Total retired OSMs   : %lld\n", retire_count);
#endif
}

void arm_simulator::activate_operation(_base_operation_ *oper,
	_OPER_ID_T *ids, void *cdg)
{
	_base_machine_ *mach = allocate_machine(oper->__get_mach_id());
	oper->__init(mach, ids, cdg);
}

_base_machine_ *arm_simulator::allocate_machine(_MACH_ID_T id)
{
	_base_machine_ *mach = machine_pool->allocate(this,id);
	mach->__set_dynid(alloc_count++);
	return mach;
}

#if 0
_base_operation_ *arm_simulator::allocate_operation(_OPER_ID_T id)
{
	return operation_pool->allocate(this,id);
}
#endif

void arm_simulator::sink_machine(_base_machine_ *mach)
{
	_base_operation_ **opers;
	unsigned ocount = mach->__get_opers(opers);

	for (unsigned i=0; i<ocount; i++)
		free_operation(opers[i]);

	retire_count++;
	mach->__retire();

	machine_pool->reclaim(mach);
}

_base_machine_ *arm_simulator::source_machine()
{
	_base_operation_ *oper = allocate_operation(_oper_disarm);
	_base_machine_ *src = allocate_machine(_mach_normal);
	oper->__init(src, NULL, NULL);

	return src;
}

_base_machine_ *arm_simulator::mach_allocator(arm_simulator *s, _MACH_ID_T id)
{
  switch (id) {
#define _MACH_DEF(_a, _b) \
	case _mach_ ## _b: return new _MACH_CLASS(_b)(s);
#include "mach_list.hpp"
	default:
      assert(0);
  }
}

_base_operation_ *arm_simulator::oper_allocator(arm_simulator *s, _OPER_ID_T id)
{
  switch (id) {
#define _OPER_DEF(_a, _b, _c) \
	case _oper_ ## _b: return new _OPER_CLASS(_b)();
#include "oper_list.hpp"
    default:
      assert(0);
  }
}

extern _UINT_T(32) syn_pc, syn_iw;

void arm_simulator::disarm_instruction(uint32_t inst, uint32_t pc)
{
	syn_iw = inst;
	syn_pc = pc+8;

	_base_operation_ *oper = allocate_operation(_oper_disarm);
	_base_machine_ *src = allocate_machine(_mach_normal);
	oper->__init(src, NULL, NULL);

	fprintf(stdout, "%8x:    %08x    %s\n", pc, inst,
		   	oper->__get_syntax().c_str());

	sink_machine(src);
}
