
#ifndef __ARMSIMUL_H__
#define __ARMSIMUL_H__

#include <list>
#include "emumem.h"
#include "include_osm.hpp"

#include "object_pool.hpp"

class arm_simulator {

	enum _machine_id {
		_mach_dummy,
		#define _MACH_DEF(_a,_b) _mach_ ## _b,
		#include "mach_list.hpp"
		_mach_total,
	};

	enum _operation_id {
		_oper_dummy,
		#define _OPER_DEF(_a,_b,_c) _oper_ ## _b,
		#include "oper_list.hpp"
		_oper_total,
	};

	public:

		/** Constructor.
		 *  @param verbose Verbose mode
		 */
		arm_simulator(bool verbose);

		/** Destructor. */
		~arm_simulator();

		/** stats output. */
		void dump_stats(FILE *fp);

		/** stats resetting. */
		void reset_stats();

		/** load the program into memory and disassemble it. */
		void disarm_program(const char *);

	public:

	   	/*  disassemble an instruciton. */
		void disarm_instruction(uint32_t, uint32_t);

		uint64_t alloc_count;	/* number of machine allocations. */
		uint64_t retire_count;	/* number of machine retires. */

		/* Allocate a machine of given id type. */
		_base_machine_ *allocate_machine(_MACH_ID_T id);

		/* Allocate an operation of given id type. */
		_base_operation_ *allocate_operation(_OPER_ID_T id) {
			return operation_pool->allocate(this,id);
		}

		 /* This is the sink of OSMs. */
		void sink_machine(_base_machine_ *mach);

		/* Get a new machine.
		 * This is the source of OSMs.
		 */
		_base_machine_ *source_machine();

		/** Activate a new operation.
		 *  @param oper The activated operation.
		 *  @param ids The ids of the machine.
		 *  @param cdg The coding of the new operation.
		 */
		void activate_operation(_base_operation_ * oper,
				_OPER_ID_T *ids, void *cdg);

	private:

		/* Free an operation object. */
		void free_operation(_base_operation_ *oper) {
			operation_pool->reclaim(oper);
		}

		static _base_machine_ *mach_allocator(arm_simulator *, _MACH_ID_T id);
		static _base_operation_ *oper_allocator(arm_simulator *, _OPER_ID_T id);

		/* storage for machine objects. */
		obj_pool<_base_machine_, _MACH_ID_T, arm_simulator,
	   		arm_simulator::mach_allocator> *machine_pool;

		/* storage for operation objects. */
		obj_pool<_base_operation_, _OPER_ID_T, arm_simulator,
	  	 	arm_simulator::oper_allocator> *operation_pool;

		/* memory management related members end/ */

		/*hardware units */

	public:

		/* memory saving the real data value */
		emulator::memory mem;
		bool verbose;

};

#endif
