
#include "more_managers_iss.hpp"
#include "simulator.hpp"
#include "device_emulate.hpp"

void fetch_manager::update_on_clock()
{
	/* a new OSM comes in */
	if (trigger) {
		current_pc = mRF->get_pc().val();
		mRF->set_pc(current_pc+4);
		trigger = false;
	}
}

void mem_ctrl_port::update_on_clock() 
{
	if (trigger) {

		assert(status==0);
		status = 1;
		addr = mMemAddr->get_value().val();
		mergeable = count > 0;
		trigger = false;
	}
	else if (count>0) {

		if (status==0 && --count>0) {
			status = 1;
			addr += 4;
		}
	}

	if (status==1) {
		if (dtlb->read(addr, size))
			status = 2;
	}

	if (status==2) {
		if (isRead && dcache->read(addr, size)) {
			if (size==4) {
				mMemRead->set_value(mem->read_word(addr));
			}
			else if (size==1) {
				mMemRead->set_value(mem->read_byte(addr));
			}
			else {
				mMemRead->set_value(mem->read_half_word(addr));
			}
			status = 0;
#if 0
			std::cerr << "read once " << std::hex << addr << " "
				   << mMemRead->get_value() << " "	;
#endif
		}
		if (!isRead && dcache->write(addr, size, mergeable)) {
			if (size==4) {
				mem->write_word(addr, mMemWrite->get_value().val());
			}
			else if (size==1) {
				mem->write_byte(addr, mMemWrite->get_value().val());
			}
			else {
				mem->write_half_word(addr, mMemWrite->get_value().val());
			}
			status = 0;
#if 0
			std::cerr << "write once " << std::hex << addr << " "
				   << mMemWrite->get_value() << " "	;
#endif
		}
	}
}

void coproc_manager::do_read()
{
	// r0 contains the device id
	dev_id_t id = mRF->get_value(0).val();

	// r1 contains the address
	dev_addr_t addr = mRF->get_value(1).val();

	dev_data_t data;

	if (has_addr) {
		if (dev_emul->receive(id, data, addr)) {
			state = 0;
			mRF->set_value(0, data);
		}
	}
	else if (dev_emul->receive(id, data)) {
		state = 0;
		mRF->set_value(0, data);
	}
}

void coproc_manager::do_write()
{
	// r0 contains the device id
	dev_id_t id = mRF->get_value(0).val();

	// r2 contains the address
	dev_addr_t addr = mRF->get_value(2).val();

	// r1 contains the value to write
	dev_data_t data = mRF->get_value(1).val();

	if (has_addr) {
		if (dev_emul->send(id, data, addr))
			state = 0;
	}
	else if (dev_emul->send(id, data)) {
		state = 0;
	}
}
