
#include "simulator.hpp"

#include "machines.hpp"
#include <vector>
#include <iomanip>
#include <iostream>

#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include "config.h"
#include "syscall.h"
#include "device_emulate.hpp"

using std::vector;

arm_simulator::arm_simulator(bool verbose) :
	running(false), debuging(false), verbose(verbose), in_fpe(false), mem()
{
	dev_emulator = new device_emulator();

	/* Create the managers */
#define MANG_DEF(cname, iname) iname = new cname(#iname);
#include "mang_list.hpp"

	/* conenct the managers. */
	mIF->connect(mRF, mReset, &mem);
	mMemCtrl->connect(mMemRead, mMemWrite, &mem, mMemAddr, mRF);
	mNewPC->connect(mRF);
	mCoProc->connect(mRF, &mem);
	mCoProc->register_dev_emulator(dev_emulator);

	init_devices(this);

	/* reset every thing */
	reset();
}

arm_simulator::~arm_simulator()
{

#define MANG_DEF(cname, iname) delete iname;
#include "mang_list.hpp"

	delete dev_emulator;

	close_devices(this);
}

void arm_simulator::reset()
{
	mem.reset();

#define MANG_DEF(cname, iname) iname->reset();
#include "mang_list.hpp"

	/* clear the value of cpsr */
	mCPSR->set_value(0);

	/* clear the value of the registers */
	for (int i=0; i<16; i++) {
		mRF->set_value(i, 0);
	}

	reset_stats();
	//brk_point = 0;
	//mmap_brk_point = MMAP_BASE;

	in_fpe = false;
}

void arm_simulator::reset_stats()
{
	inst_count = 0;
}

void arm_simulator::dump_stats(FILE *fp)
{
	fprintf(fp, "Total instructions : ");
	dump_int64(inst_count, fp);
	fprintf(fp, "\n");

	fprintf(stderr, "Total memory pages allocated : %d\n",
		 mem.get_page_count());
}

void arm_simulator::run(UInt64 maxInst)
{
	_MACH_CLASS(fetch_normal) *mach = new _MACH_CLASS(fetch_normal)(this);
	mIF->set_pc(mRF->get_pc().val());

	running = true;
	while(running && inst_count<=maxInst) {
//		for (int i=0;i<15;i++) fprintf(stderr, "%x ", mRF->get_value(i).val());
//		fprintf(stderr, "\n");
		mach->__activate();
		inst_count++;
	}
	delete mach;
}

