
#ifndef __ARMSIMUL_H__
#define __ARMSIMUL_H__

#include "emumem.h"

#include "more_managers_iss.hpp"

/* forward declaration */
class device_emulator;

class arm_simulator {

	enum _machine_id {
		_mach_dummy,
		#define _MACH_DEF(_a,_b) _mach_ ## _b,
		#include "mach_list.hpp"
		_mach_total,
	};

	public:

		/** Constructor.
		 *  @param verbose Verbose mode
		 */
		arm_simulator(bool verbose);

		/** Destructor. */
		~arm_simulator();

		/** simulate for n cycles.
		 *  @param n The maximum number of cycles to simulate (may exit).
		 */
		void run(UInt64 n);

		/** stop running. 
		 *  Called by the signal handler or syscall interpreter.
		 */
		void stop() {running = false;}

		/** See if the simulator is running.
		 */
		bool is_running() const {return running;}

		/** Stop debuging, useless here.
		 *  Kept only to satisfy syscall.cpp.
		 */
		void stop_debug() {debuging = false;}

		/** See if in debugging mode, useless here.
		 *  Kept only to satisfy syscall.cpp.
		 */
		bool is_debugging() {return debuging;}

		/** stats output. */
		void dump_stats(FILE *fp);

		/** stats resetting. */
		void reset_stats();

		/* set brk point for syscall interpretation. */
		void set_brk(arm_addr_t addr) {brk_point = addr;}
		void set_mmap_brk(arm_addr_t addr) {mmap_brk_point = addr;}

		/** load the program into memory. */
		void load_program(const char *, int argc, char *argv[], char *env[]);

		/** load fpe emulation library into memory. */
		void load_fpe(const char *);

		/** reset */
		void reset();

		/** Get the value of a register. For debugging use. */
		word_t read_gpr(int ind) const {
			return mRF->get_value(ind).val();
		}

		/** These two hooks for the syscall interpretor. */
		void write_gpr(int ind, word_t val) {
			mRF->set_value(ind, val);
		}

		void write_gpr2(int ind, word_t val) {
			mRF->set_value(ind, val);
		}

		uint64_t get_inst_count() const {return inst_count;}

		word_t read_cpsr() {return mCPSR->get_value().val();}
		void write_cpsr(word_t val) {mCPSR->set_value(val);}

#if 0
		word_t read_spsr() {return my_regs.spsr;}

		void write_spsr(word_t val) {my_regs.spsr=val;}
#endif


	private:
		/* status flags */
		bool running;
		bool debuging;

	public:
		/* syscall data */
		bool verbose;
		bool emulate_syscall;
		target_addr_t brk_point;
		target_addr_t mmap_brk_point;

		/* emulating fpe */
		bool in_fpe;

		/* memory management related members */

	public:

		uint64_t inst_count;	/* number of machine allocations. */


		/* memory management related members end/ */

		/*hardware units */

		/* the token managers */
#define MANG_DEF(cname, iname) cname *iname;
#include "mang_list.hpp"

	public:
		device_emulator *dev_emulator;

		/* memory saving the real data value */
		emulator::memory mem;

};

#endif
