
#include "device_emulate.hpp"
#include "simulator.hpp"

using std::map;

device_emulator::device_emulator() : last_id(0), last_dev(NULL)
{
}

device_emulator::~device_emulator()
{
}

bool device_emulator::register_device(dev_id_t id, sima_device *dev)
{
	if (devs.find(id)!=devs.end()) return false;
	devs[id] = dev;
	return true;
}

bool device_emulator::unregister_device(dev_id_t id)
{
	if (devs.find(id)==devs.end()) return false;
	devs.erase(id);
	if (last_id==id) last_dev=NULL;
	return true;
}

bool device_emulator::send(dev_id_t id, dev_data_t val, dev_addr_t addr)
{
	/* check cached result*/
	if (last_dev && id==last_id) return last_dev->write(val, addr);

	map<dev_id_t, sima_device *>::iterator dev_it;

	dev_it = devs.find(id);
	assert(dev_it!=devs.end());

	last_id = id;
	last_dev = (*dev_it).second;

	return last_dev->write(val, addr);
}

bool device_emulator::receive(dev_id_t id, dev_data_t &val, dev_addr_t addr)
{
	/* check cached device*/
	if (last_dev && id==last_id) return last_dev->read(val, addr);

	map<dev_id_t, sima_device *>::iterator dev_it;

	dev_it = devs.find(id);
	assert(dev_it!=devs.end());

	last_id = id;
	last_dev = (*dev_it).second;

	return last_dev->read(val, addr);
}

#include "dev_test.hpp"

void init_devices(arm_simulator *sim)
{
	//create and register your devices here
	test_device *test = new test_device();
	sim->dev_emulator->register_device(0, test);
}

void close_devices(arm_simulator *sim)
{
	//close your devices here
}
