
#ifndef __LOADER_HPP__
#define __LOADER_HPP__

#include "bittypes.h"

typedef struct __attribute__((packed))
{
	uint16_t version_id; 
	uint16_t num_sections;	// Number of sections
	uint32_t time_stamp;	// Time (seconds) from Jan 1, 1970
	uint32_t symbolP;		// Pointer to symbol table
	uint32_t num_symbols; 	// Number of symbols in symbol table
	uint16_t num_bytes_opt;	// Number of bytes in Optional Header (0 or 28)
	uint16_t flags;			// Flags
	/*
	 * Reference: C54 Assembly Language Tools Users Guide (Appendix A)
	 * F_RELFLG	 0001h	Relocation information was stripped from the file.
	 * F_EXEC	 0002h	The file is relocatable (it contains no unresolved
	 *					external references).
	 * F_LNNO	 0004h	Line numbers were stripped from the file.
	 * F_LSYMS 	 0010h	Local symbols were stripped from the file.
	 * F_LENDIAN 0100h	The file has the byte ordering used by 'C54x de-
	 *					vices (16 bits per word, least significant byte first)
	 * F_SYMMERGE 1000h	Duplicate symbols were removed.
	 */
	uint16_t target_id; // Magic number. 0x108 for SunOS and 0x801 for x86
	// Can be used to see if Big Endian (0x108) or Little Endian (0x801)

} CoffHeader;

typedef struct __attribute__((packed))
{
	char	 s_name[8];	// 8-character null padded section name
	uint32_t s_paddr;	// Physical address of section
	uint32_t s_vaddr;	// Virtual address of section.	always same as above?
	uint32_t s_size;	// Section size in bytes
	uint32_t s_scnptr;	// File pointer to raw data. =0 for non-initialized sections ?

	uint32_t s_relptr;	// File pointer to relocation entries
	uint32_t s_lnnoptr; // File pointer to line number entries
	uint16_t s_nreloc;	// Number of relocation	entries
	uint16_t s_nlnno;	// Number of line number entries
	uint32_t s_flags;	// Flags. only nonzero for .text section?
	char	 bogus[8];	// not in the spec, but needed for c54
} SecHeader;

#if 0
typedef struct 
{
	char					 s_name[8];	 // 8-character null padded section name
	uint32_t			 s_paddr;	 // Physical address of section
	uint32_t			 s_vaddr;	 // Virtual address of section.	always same as above?
	uint32_t s_size;		// Section size in bytes
	uint32_t			 s_scnptr;	// File pointer to raw data. =0 for non-initialized sections ?

	uint32_t			 s_relptr;	// File pointer to relocation entries
	uint32_t			 s_lnnoptr; // File pointer to line number entries
	uint16_t s_nreloc;	// Number of relocation	entries
	uint16_t s_nlnno;	 // Number of line number entries
	long			 s_flags;	 // Flags. only nonzero for .text section?
} SecHeaderBU;	//Binutil header

typedef union
{
	struct _section_header_binutil binutil;
	struct _section_header_ti ti;
} SecHeader;
#endif


typedef struct
{
	uint16_t magic_num;	// Magic number. 0x108 for SunOS and 0x801 
	// for x86. Can be used to see if Big Endian (0x108) or Little Endian (0x801)
	
	uint16_t version;
	uint32_t size_executable; // size (Words) of executable
	uint32_t size_initialized; // size (Words) of initialized sections
	uint32_t size_uninitialized; // size (Words) of uninitialized sections
	uint32_t start_address; // entry location
	uint32_t start_executable; // start of executable
	uint32_t start_initialized; // start of initialized section
} CoffOptHeader;

#endif
