
#ifndef __C54XSIMUL_H__
#define __C54XSIMUL_H__

#include <list>
#include "emumem.h"
#include "bittypes.h"

#include "object_pool.hpp"

class c54x_simulator {

	public:

		/** Constructor.
		 *  @param verbose Verbose mode
		 */
		c54x_simulator(bool verbose) : verbose(verbose) {}

		/** Destructor. */
		~c54x_simulator();

		/** simulate for n cycles.
		 *  @param n The maximum number of cycles to simulate (may exit).
		 */
		void run(uint64_t n);

		/** simulate one cycle. */
		void clock_tick();

		/** stop running. 
		 *  Called by the signal handler or syscall interpreter.
		 */
		void stop() {running = false;}

		/** stats output. */
		void dump_stats(FILE *fp);

		/** stats resetting. */
		void reset_stats();

		/** load the program into memory. */
		void load_program(const char *);

		/** reset */
		void reset();

		uint64_t get_cycle_count() const {return cycle_count;}

	private:

		/* status flags */
		bool running;

		uint64_t cycle_count;	/* number of simulation cycles*/

#if 0
		std::list<_base_machine_ *> work_list;
		std::list<_base_machine_ *>::iterator work_it, ins_it;
#endif

	public:

		uint64_t alloc_count;	/* number of machine allocations. */
		uint64_t retire_count;	/* number of machine retires. */


#if 0
		/* Allocate a machine of given id type. */
		_base_machine_ *allocate_machine(_MACH_ID_T id);

		/* Allocate an operation of given id type. */
		_base_operation_ *allocate_operation(_OPER_ID_T id) {
			return operation_pool->allocate(this,id);
		}

		 /* This is the sink of OSMs. */
		void sink_machine(_base_machine_ *mach);

		/* Get a new machine.
		 * This is the source of OSMs.
		 */
		_base_machine_ *source_machine();

		/** Activate a new operation.
		 *  @param oper The activated operation.
		 *  @param ids The ids of the machine.
		 *  @param cdg The coding of the new operation.
		 */
		void activate_operation(_base_operation_ * oper,
				_OPER_ID_T *ids, void *cdg);

	private:

		/* Free an operation object. */
		void free_operation(_base_operation_ *oper) {
			operation_pool->reclaim(oper);
		}

		static _base_machine_ *mach_allocator(c54x_simulator *, _MACH_ID_T id);
		static _base_operation_ *oper_allocator(c54x_simulator *, _OPER_ID_T id);

		/* storage for machine objects. */
		obj_pool<_base_machine_, _MACH_ID_T, c54x_simulator,
	   		c54x_simulator::mach_allocator> *machine_pool;

		/* storage for operation objects. */
		obj_pool<_base_operation_, _OPER_ID_T, c54x_simulator,
	  	 	c54x_simulator::oper_allocator> *operation_pool;
#endif

		/* memory management related members end/ */

	public:

		/* memory saving the real data value */
		emulator::ProgMemory pmem;
		emulator::DataMemory dmem;

		target_addr_t prog_base;

		bool verbose;
};


#endif
