
#include <cstdio>
#include <csignal>
#include <cstdlib>
#include "simulator.hpp"

#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#endif


static void usage(char *fname)
{
	char *cp;
	if ((cp = rindex(fname, '/'))!=NULL) cp++;
	else cp = fname;
	fprintf(stderr, 
		"usage : %s [-n] [-h]\n"
		"  -n : index into the series, starting from 0\n"
		"  -h : print this message and quit\n", cp);
}

simple_simulator *sima;

uint64_t ato_uint64(char *input)
{
	char *p = input;
	uint64_t val = 0;
	while (*p!=0) {
		val = val*10+(*p-'0');
		p++;
	}
	return val;
}

void sigint_handler(int signum)
{
	if (sima->is_running()) {
		sima->stop();
	}
	else exit(0);
}

int main(int argc, char *argv[], char *envp[])
{
	int i;
	bool verbose = false;
	uint64_t max_cnum = 100;
	void (*prev_sigint_handler)(int);

	for(i = 1; i < argc; i++)
	{
		if(strcmp(argv[i], "-n") == 0) max_cnum = ato_uint64(argv[++i]); else
		if(strcmp(argv[i], "-h") == 0) {usage(argv[0]); return 0;} else
		{
			usage(argv[0]);
			exit(1);
		}

	}
	
	sima = new simple_simulator(verbose);
	prev_sigint_handler = signal(SIGINT, sigint_handler); 

	sima->run(max_cnum);
	if(prev_sigint_handler != SIG_ERR) signal(SIGINT, SIG_DFL);

	fprintf(stderr, "n=%lld\n", max_cnum);
	sima->dump_stats(stderr);

	return 0;
}

