
#include "simulator.hpp"

#include "machines.hpp"
#include <vector>
#include <iostream>

#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include "config.h"

using std::vector;
using std::cerr;
using std::endl;

hip_simulator::hip_simulator(bool verbose) :
	verbose(verbose), running(false), pmem(), dmem()
{
	/* Create the managers */
#define MANG_DEF(cname, iname) iname = new cname(#iname);
#include "mang_list.hpp"

	/* conenct the managers. */
	
	mFetch->connect(mBranch, &pmem);
	mMemCtrl1->connect(mMemIData1, &dmem, mMemAddr1);
	mMemCtrl2->connect(mMemIData2, &dmem, mMemAddr2);
	mLReg->connect(mIReg);
	mAccReg->connect(mIReg);

	/* reset every thing */
	reset();
}

hip_simulator::~hip_simulator()
{
#define MANG_DEF(cname, iname) delete iname;
#include "mang_list.hpp"

}

void hip_simulator::reset()
{
	pmem.reset();
	dmem.reset();

#define MANG_DEF(cname, iname) iname->reset();
#include "mang_list.hpp"

	reset_stats();
}

void hip_simulator::reset_stats()
{
	inst_count = 0;
}

void hip_simulator::dump_stats(FILE *fp)
{
	fprintf(fp, "Total instructions : ");
	dump_int64(inst_count, fp);
	fprintf(stderr, "\n");
}


void hip_simulator::run(UInt64 maxInst)
{
	_MACH_CLASS(fetcher_normal) *mach = new _MACH_CLASS(fetcher_normal)(this);  

	running = true;
	while(running && inst_count<=maxInst) {
		mach->__activate();
		inst_count++;
	}
	delete mach;
}

