
#ifndef __PLXSIMUL_H__
#define __PLXSIMUL_H__

#include <iostream>
#include "emumem.h"
#include "more_managers_iss.hpp"

class hip_simulator {

	enum _machine_id {
		_mach_dummy,
		#define _MACH_DEF(_a,_b) _mach_ ## _b,
		#include "mach_list.hpp"
		_mach_total,
	};

	public:

		/** Constructor.
		 *  @param verbose Verbose mode
		 */
		hip_simulator(bool verbose);

		/** Destructor. */
		~hip_simulator();

		/** simulate for n cycles.
		 *  @param n The maximum number of cycles to simulate (may exit).
		 */
		void run(UInt64 n);

		/** stop running. 
		 *  Called by the signal handler or syscall interpreter.
		 */
		void stop() {running = false;}

		/** See if the simulator is running.
		 */
		bool is_running() const {return running;}

		/** stats output. */
		void dump_stats(FILE *fp);

		/** stats resetting. */
		void reset_stats();

		/** load the program into memory. */
		void load_program(const char *, int argc, char *argv[], char *env[]);

		/** reset */
		void reset();

#if 0
		/** Get the value of a register. For debugging use. */
		word_t read_gpr(int ind) const {
			return mRF->get_value(ind).val();
		}

		/** These two hooks for the syscall interpretor. */
		void write_gpr(int ind, word_t val) {
			mRF->set_value(ind, val);
		}
#endif
		uint64_t get_cycle_count() const {return inst_count;}


	private:
		bool verbose;

		/* status flags */
		bool running;

		uint64_t inst_count;	/* number of simulation cycles*/

	public:

		/* caches, TLBs & bus interface unit, modeling timing only */

		/* the token managers */
#define MANG_DEF(cname, iname) cname *iname;
#include "mang_list.hpp"

	public:

		/* memory saving the real data value */
        ProgMemory pmem;
        DataMemory dmem;

		target_addr_t prog_base;

};

#endif
