
#ifndef __PLXSIMUL_H__
#define __PLXSIMUL_H__

#include <list>
#include "emumem.h"

#include "more_managers.hpp"
#include "object_pool.hpp"

class hip_simulator {

	enum _machine_id {
		_mach_dummy,
		#define _MACH_DEF(_a,_b) _mach_ ## _b,
		#include "mach_list.hpp"
		_mach_total,
	};

	public:

		/** Constructor.
		 *  @param verbose Verbose mode
		 */
		hip_simulator(bool verbose);

		/** Destructor. */
		~hip_simulator();

		/** simulate for n cycles.
		 *  @param n The maximum number of cycles to simulate (may exit).
		 */
		void run(UInt64 n);

		/** simulate one cycle. */
		void clock_tick();

		/** stop running. 
		 *  Called by the signal handler or syscall interpreter.
		 */
		void stop() {running = false;}

		/** See if the simulator is running.
		 */
		bool is_running() const {return running;}

		/** stats output. */
		void dump_stats(FILE *fp);

		/** stats resetting. */
		void reset_stats();

		/** load the program into memory. */
		void load_program(const char *, int argc, char *argv[], char *env[]);

		/** reset */
		void reset();

#if 0
		/** Get the value of a register. For debugging use. */
		word_t read_gpr(int ind) const {
			return mRF->get_value(ind).val();
		}

		/** These two hooks for the syscall interpretor. */
		void write_gpr(int ind, word_t val) {
			mRF->set_value(ind, val);
		}
#endif

		uint64_t get_cycle_count() const {return cycle_count;}


	private:
		bool verbose;

		/* status flags */
		bool running;

		uint64_t cycle_count;	/* number of simulation cycles*/

		std::list<_opt_machine_ *> work_list;
		std::list<_opt_machine_ *>::iterator work_it, ins_it;

	public:

		uint64_t alloc_count;	/* number of machine allocations. */
		uint64_t retire_count;	/* number of machine retires. */


		/* Allocate a machine of given id type. */
		_opt_machine_ *allocate_machine(_MACH_ID_T id);

		 /* This is the sink of OSMs. */
		void sink_machine(_opt_machine_ *mach);

		/* Get a new machine.
		 * This is the source of OSMs.
		 */
		_opt_machine_ *source_machine();

		/** Activate a new machine.
		 *  @param id The id of the machine.
		 */
		void activate_machine(_MACH_ID_T id);

	private:

		static _opt_machine_ *mach_allocator(hip_simulator *, _MACH_ID_T id);

		/* storage for machine objects. */
		obj_pool<_opt_machine_, _MACH_ID_T, hip_simulator,
	   		hip_simulator::mach_allocator> *machine_pool;

		/* memory management related members end/ */

		/*hardware units */

	public:

		/* caches, TLBs & bus interface unit, modeling timing only */

		/* the token managers */
#define MANG_DEF(cname, iname) cname *iname;
#include "mang_list.hpp"

	public:

		/* memory saving the real data value */
        ProgMemory pmem;
        DataMemory dmem;

		target_addr_t prog_base;

};

#endif
