#include "loader.h"

#include <cstdio>
#include <cstdlib>
#include <cctype>
#include <cstring>

unsigned int bin2hex(char *val)
{
	unsigned ret = 0;
	int len = strlen(val);
	for (int i=0; i<len; i++) {
		if (val[i]==' ' || val[i]=='\t') {
			len = i;
			break;
		}
	}

	for (int i=len-1; i>=0; i--) {
		ret |= (val[i]-'0') << (len-1-i);
	}
	return ret;
}

void simulator::load_program(const char *filename, 
		int argc, char *argv[], char *envp[])
{
	//code to be filled
	FILE *fp;
	unsigned int lineno = 0;
	unsigned int prog_addr;
	unsigned int data_addr;
	bool in_pmem = false;
	bool in_dmem = false;

	fp = fopen(filename, "rt");
	if (fp==NULL) {
		fprintf(stderr, "Error openning file %s\n", filename);
		exit(1);
	}

	char buf[1024];
	while (fgets(buf, 1000, fp)!=NULL) {

		lineno++;

		/* remove the return character */
		if (buf[strlen(buf)-1]=='\n') 
			buf[strlen(buf)-1] = 0;

		/* tokenize the line */
		char *ptr = strtok(buf, " \t");

		if (ptr==NULL) {
			// empty line do nothing.
			continue;
		}
		else if (strcmp(ptr, "data")==0) {

			char *ptr2 = strtok(NULL, " \t");

			if (ptr2==NULL) {
				fprintf(stderr, "Error reading file, near line %d\n", lineno);
				exit(1);
			}
			/* beginning of data segment */
			else if (strcmp(ptr2, "segment")==0) {


				in_dmem = true;
				in_pmem = false;

				char *ptr3 = strtok(NULL, " \t");
				data_addr = strtoul(ptr3, NULL, 10);
				fprintf(stderr, "Reading data segment at %04x\n", data_addr);
			}
			/* oddly, someone use data as function name */
			else if (isdigit(ptr2[0])) {


				in_pmem = true;
				in_dmem = false;
				prog_addr = strtoul(ptr2, NULL, 10);
				fprintf(stderr, "Reading function data:%04x\n", prog_addr);
			}
		}
		/* this has to be the beginning of a function. */
		else if (isdigit(ptr[0])) {
			unsigned val = bin2hex(ptr);

			if (in_pmem) {
				pmem.write_word(prog_addr<<1, val);
			//	fprintf(stderr, "%04x:%04x\n", prog_addr, val);
				prog_addr += 1;
			}
			else if (in_dmem) {
				dmem.write_word(data_addr<<1, val);
				data_addr += 1;
			}
			else {
				fprintf(stderr, "Error reading file, near line %d\n", lineno);
				exit(1);
			}
		}
		/* this must be function beginning. */
		else {
			in_pmem = true;
			in_dmem = false;

			char *ptr2 = strtok(NULL, " \t");

			if (ptr2==NULL) {
				fprintf(stderr, "Error reading file, near line %d\n", lineno);
				exit(1);
			}
			prog_addr = strtoul(ptr2, NULL, 10);
			fprintf(stderr, "Reading function %s:%04x\n", ptr, prog_addr);

			if (strcmp(ptr, "main")==0) {
				// set program counter here
				mFetch->set_pc(prog_addr);
			}
		}
	}

	fclose(fp);
}
