
#ifndef __EMU_MEMORY_H__
#define __EMU_MEMORY_H__

#include "misc.h"
#include <cstring>

const unsigned int HIP_MEMORY_SIZE = 65536;

class memory
{
	public:

		/* constructor */
		memory();

		/* destructor */
		~memory();

		/* free all memory pages */
		void reset();

		/* read/write operations */
		byte_t read_byte(target_addr_t addr) {
			return data[addr];
		}

		void write_byte(target_addr_t addr, byte_t value) {
			data[addr] = value;
		}

		word_t read_word(target_addr_t addr) {
#if WORDS_BIGENDIAN==TARGET_LITTLE_ENDIAN
			return swap_word(*((word_t *)(data+addr)));
#else
			return *((word_t *)(data+addr));
#endif
		}

		void write_word(target_addr_t addr, word_t value) {
#if WORDS_BIGENDIAN==TARGET_LITTLE_ENDIAN
			*((word_t *)(data+addr)) = swap_word(value);
#else
			*((word_t *)(data+addr)) = value;
#endif
		}

		/* write operations */
		dword_t read_dword(target_addr_t addr) {
#if WORDS_BIGENDIAN==TARGET_LITTLE_ENDIAN
			return swap_dword(*((dword_t *)(data+addr)));
#else
			return *((dword_t *)(data+addr));
#endif
		}

		void write_dword(target_addr_t addr, dword_t value) {
#if WORDS_BIGENDIAN==TARGET_LITTLE_ENDIAN
			*((dword_t *)(data+addr)) = swap_dword(value);
#else
			*((dword_t *)(data+addr)) = value;
#endif
		}

		/* block read, write */
		void set_block(target_addr_t addr, byte_t value, unsigned int size);
		void read_block(void *buf, target_addr_t addr, unsigned int size);
		void write_block(target_addr_t, void *buf, unsigned int size);

	protected:

		static word_t swap_word(word_t val) {
			return (val>>8) | (val<<8);
		}

		static dword_t swap_dword(dword_t val) {
			return (val>>24) | ((val>>8)&0xFF00) |
				   ((val&0xFF00)<<8) | (val<<24) ;
		}

		byte_t *data;
};

typedef memory ProgMemory;
typedef memory DataMemory;

#endif





