
#ifndef __NS_MANAGER_H_
#define __NS_MANAGER_H_

#include <queue>
#include <utility>
#include <iostream>
using std::cerr;
using std::cout;
using std::endl;
#include <stdio.h>
#include "managers.hpp"
#include "ns_parm.hpp"


/* include the arm simulator files */
namespace simulator {
	class arm_simulator;
};

#include "arm_if_dev.h"
/** simple resource manager. */
typedef unit_manager<int> simple_resource;



#define BUS_NUM_INSTS NETWORK_SIZE
#define BUS_ARBITER_SIZE BUS_NUM_INSTS
#define BUS_PROB PROBABILITY
#define BUS_BUFFER_DEPTH BUFFER_DEPTH
#define GENERIC_READ 0
#define GENERIC_WRITE 1
#define BUS_READ 0
#define BUS_WRITE 1
#define BUS_BURST_READ 3
#define BUS_BURST_WRITE BUS_WRITE
#define BUS_LOCK_READ 5
#define BUS_LOCK_WRITE 6
#define BUS_UNLOCK_READ 7
#define BUS_UNLOCK_WRITE 8
#define BUS_NO_LOCK 0
#define BUS_LOCK 1
#define BUS_UNLOCK 2
#define BUS_LOCKED 2
#define BUS_IDLE 0
#define BUS_TIMEOUT 40
#define BUS_PRIMARY 1
#define BUS_SECONDARY 2
#define BUS_PRIMARY_REL_READ 3
#define BUS_SECONDARY_REL_READ 4
#define BUS_DACK_READ 5
#define BUS_PRIMARY_REL_WRITE 6
#define BUS_SECONDARY_REL_WRITE 7
#define BUS_DACK_WRITE 8
#define BUS_DELOCK 9
#define BUS_RELOCK 10
#define BUS_SAME_MASTER_SLAVE 11
#define BFL_MASTER_PREFIX "cc_ex"
#define BFL_SLAVE_PREFIX "cc_sl"

/* config parameter for the cc_bus_arbiter deterimining if
 * it would support address pipelining
 */

#define CC_BUS_ARBITER_SINGLE 0
#define CC_BUS_ARBITER_PIPELINE 1

/* define how many instance per class would be instantiated
 * currently we use each manager per node, so NUM_INSTS = 1 
 */
#define NUM_INSTS 1

#define VC_IDLE 0
#define VC_ACTIVE 1
#define VC_RID 2 
 
#define HEAD 1
#define BODY 2
#define TAIL 3

/* define how many the queue of the fetch would hold for each 
 * simulation run. A backlog bigger than this would cause the 
 * code to break. 
 */

#define MAX_TOKEN_COUNT  100000

extern unsigned int myrand();        /* returns a random integer */
//extern void myrand_init( unsigned int );     /* initialize the generator */
//extern void calc_next_src_tlb();


template<class S, int count>
class ns_serialize_manager {//

	public:

		typedef class _token_<ns_serialize_manager<S, count> > TOKEN_T;

		ns_serialize_manager(const std::string& name) : name(name) {
                  real_token = new TOKEN_T[count](this);
                  token_pool = new std::queue<TOKEN_T *>; 
                  token_queue = new std::queue<TOKEN_T *>;
                  for (int j=0;j<count;j++) 
                        token_pool->push(real_token+j);    
                         
                  }

		~ns_serialize_manager() {
				
		}

		bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
                   if(token_queue->size() >= count){
                        return false;
                   }else{        
		        return true;
                        }
		}

		bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
                  return  true;
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
		  if (token == token_queue->front()){
                    return true;
                  }else{
                    return false;
                  }
		}

		TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
                  TOKEN_T * token;
                  token = token_pool->front();
                  token_pool->pop();
                  token_queue->push(token);
                  token->set_used(obj);
                  return  token;
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		
                }


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
                  
                  token_queue->pop();
                  token_pool->push(token);
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
		  token_queue->pop();
                  token_pool->push(token);
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                  
                }

	

	private:

		std::string name;                
		TOKEN_T *real_token;
	        std::queue<TOKEN_T*> *token_queue;
                std::queue<TOKEN_T*> *token_pool;
};

template<class S>
class ns_timer_manager {//

	public:

		typedef class _token_<ns_timer_manager<S> > TOKEN_T;

		ns_timer_manager(const std::string& name) : name(name) {
                  timer_count = 0;
                  token = new TOKEN_T(this);
                }

		~ns_timer_manager() {
				
		}

		bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
                  if (timer_count ==0){
                    return true;
                  }else{
                    return false;
                  }
		}

		bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
                  return  true;
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
		  if (timer_count==0){
                    return true;
                  }else{
                    //cerr << " now timer =" << timer_count << endl;
                    return false;
                  }
		}

		TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
                  
                  assert(ind.val()>0);
                  timer_count = ind.val();
                  token->set_used(obj);
                  return  token;
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		
                }


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                  if(timer_count >0){
                    timer_count --;
                  }
                }

	

	private:

		std::string name;
                unsigned timer_count;	
		TOKEN_T *token;
	
};

template<class S>
class ns_reduced_timer_manager {//

	public:

		typedef class _token_<ns_reduced_timer_manager<S> > TOKEN_T;

		ns_reduced_timer_manager(const std::string& name) : name(name) {
                  timer_count = 0;
                  token = new TOKEN_T(this);
                }

		~ns_reduced_timer_manager() {
				
		}

		bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
		  return true;
		}

		bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
                  return  true;
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
		  if (timer_count==0){
                    return true;
                  }else{
                    return false;
                  }
		}

		TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
                  
                  assert(ind.val()>0);
                  timer_count = ind.val()-1;
                  token->set_used(obj);
                  return  token;
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		
                }


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                  if(timer_count >0){
                    timer_count --;
                  }
                }

	

	private:

		std::string name;
                unsigned timer_count;	
		TOKEN_T *token;
	
};

template<class S>
class ns_phase_resource_manager{//

	public:

		typedef class _token_<ns_phase_resource_manager<S> > TOKEN_T;

		ns_phase_resource_manager(const std::string& name) : name(name) {
                  intend_free = false;
                  timer_count = 0;
                  token = new TOKEN_T(this);
                }

		~ns_phase_resource_manager() {
		  delete token;
		}

		bool token_free_for_alloc(_base_machine_ *obj) {
		  return token->is_free();
		}

		bool token_free_for_inquire(_base_machine_ *obj) {
                  return token->is_free() || obj==token->get_tenant();
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
		  return true;
		}

		TOKEN_T *allocate_token(_base_machine_ *obj) {
                  
                  token->set_used(obj);
                  return token;
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		
                }


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				intend_free = true;
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                  
                    timer_count ++;
                    if (timer_count % 3==0 && intend_free ){
                        token->set_unused();
                        intend_free = false;
                       }
                  }
	

	private:

		std::string name;
                bool intend_free;
                unsigned timer_count;	
		TOKEN_T *token;
	
};

template<class S, int count>
class ns_phase_risingedge_manager{//

	public:

		typedef class _token_<ns_phase_risingedge_manager<S, count> > TOKEN_T;

		ns_phase_risingedge_manager(const std::string& name) : name(name) {
                  
                  timer_count = 0;
                  token = new TOKEN_T(this);
                }

		~ns_phase_risingedge_manager() {
		  delete token;
		}

		bool token_free_for_alloc(_base_machine_ *obj) {
                  if (timer_count % 3 == count){
		        return true;
                  }else{
                        return false;
                  }
		}

		bool token_free_for_inquire(_base_machine_ *obj) {
                  return true;
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
		  return true;
		}

		TOKEN_T *allocate_token(_base_machine_ *obj) {
                  
                  token->set_used(obj);
                  return token;
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		
                }


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                  
                    timer_count ++;
                  
                 }
	

	private:

		std::string name;
                bool intend_free;
                unsigned timer_count;	
		TOKEN_T *token;
	
};


template<class S>
class ns_prob_resource_manager {//

	public:

		typedef class _token_<ns_prob_resource_manager<S> > TOKEN_T;

		ns_prob_resource_manager(const std::string& name) : name(name) {
                  timer_count = 0;
                  token = new TOKEN_T(this);
                }

		~ns_prob_resource_manager() {
				
		}

		bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
              
                 switch(BUS_CONFIG){
                 case 521:
                 case 523:
                   //non-delayed-ack case
                   return true;
                 case 522: 
                 case 524:
                   //delayed-ack case
                  if (timer_count ==1) // should be a probalistic statement
                    { //now only the second request is considered a delayed ack
                         return false;
                     }else{
		         return true;
                     }             
                 default:
                   return true;
                  break;
                 }
		}

		bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
                  return  true;
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
                  return true;
		}

		TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
                  
                  timer_count ++;
                  token->set_used(obj);
                  return  token;
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		
                }


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                 
                }

	

	private:

		std::string name;
                unsigned timer_count;	
		TOKEN_T *token;
	
};

template<class S, int count, int nTotal>
class ns_xbar_manager {

	public:

		typedef class _token_<ns_xbar_manager<S,count, nTotal> > TOKEN_T;

		ns_xbar_manager(const std::string& name) : name(name) {
                  
                  tokens = new TOKEN_T [count*nTotal](this);
                  for (int j=0;j<count;j++){
			
			for (int i=0;i<nTotal;i++) tokens[(j*nTotal)+i].set_unused();
		  }
                }

		~ns_xbar_manager() {
		      
                      delete [] tokens;
			
		}

		bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
			return true;
                   //return tokens[(((unsigned)(ind.first).val())*nTotal)+((unsigned)(ind.second).val())].is_free();
		}

		bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
			return tokens[(unsigned)(ind.first).val()*nTotal+(ind.second).val()].is_free() ||
				   obj==tokens[(ind.first).val()*nTotal+(ind.second).val()].get_tenant();
		}

		bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
			return true;
		}

		TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
			tokens[(ind.first).val()*nTotal+(ind.second).val()].set_used(obj);
			return &(tokens[(ind.first).val()*nTotal+(ind.second).val()]);
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
		}


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {}

	

	private:

		std::string name;

	
		TOKEN_T *tokens;
	
};


template<class S, class T, int count, int nTotal, int vc_size>
class ns_vc_arbiter_manager {

	public:

		typedef class _token_<ns_vc_arbiter_manager<S, T, count, nTotal, vc_size> > TOKEN_T;

		ns_vc_arbiter_manager(const std::string& name) : name(name) {
                  
                  tokens = new TOKEN_T [count*nTotal*vc_size](this);

                  for (int j=0;j<count*nTotal*vc_size;j++){
		     
                            tokens[j].set_unused();
		  }

               
                  /* init the port_list */
                  for (int i =0; i < count; i++){
                      for (int j =0; j < nTotal; j++){
                        output_score[i][j] = false;
                        input_score[i][j] = false;
                        for (int k =0; k < vc_size; k++){
                        port_list[i][j][k] = -1;
                        request_port[i][j][k] = -1;
                        request_flit_type[i][j][k] = -1;
                        vc_status[i][j][k] = VC_IDLE;
                        vc_status_inv[i][j][k] = VC_IDLE;
                        vc_table[i][j][k].first = -1;
                        vc_table[i][j][k].second = -1;
                        vc_table_inv[i][j][k].first = -1;
                        vc_table_inv[i][j][k].second = -1;
                        credit_score[i][j][k] = BUFFER_DEPTH; 
                    }
                   }
                  }

                  for (int j =0; j < count; j++){
                    cursor_last_[j] = 0;
                    for (int k =0; k < nTotal; k++){
                      cursor_vc_last_[j][k] = 0;
                    }
                  }  
                }

		~ns_vc_arbiter_manager() {
		      
                      delete [] tokens;
			
		}

		bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
		  unsigned ins_num, port_num,vc_num;
                  ins_num = (unsigned)(ind.first).val();
                  port_num= (unsigned)(ind.second).val();
                  vc_num= (unsigned)(ind.third).val();
                  assert(ins_num < count);
                  assert(port_num < nTotal);
                  assert(vc_num< vc_size);
                  return (request_port[ins_num][port_num][vc_num]==-1);
		}

		bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
                  return true;
		}

               bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
                 unsigned offset, ins_num, port_num,vc_num;
                 offset = unsigned (token - tokens);
                 vc_num = offset % vc_size;
                 ins_num = ((offset - vc_num)/vc_size)/nTotal;
                 port_num = ((offset - vc_num)/vc_size)%nTotal;
                 //std::cout << "reclaim inst=" << ins_num << " buf_num= " << port_num << " i_vc = " << vc_num << " out vc ="<< port_list[ins_num][port_num][vc_num] <<std::endl;
                 if (port_list[ins_num][port_num][vc_num] > -1){
                   return true;
                 }else{
                   return false;
                 }
  }

		TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
                        unsigned first, second, third, fourth, fifth;
                        /* instance number */
                        first = (ind.first).val();
                        /* input buffer number */
                        second = (ind.second).val();
                        /* vc number */
                        third = (ind.third).val();
                        /* output buffer number */
                        fourth = (ind.fourth).val();
                        fifth = (ind.fifth).val();
                        assert(first < count);
                        assert(second < nTotal);
                        assert(third < vc_size);
                        assert(fourth < nTotal);
                        if (vc_size > 1)
                        assert(fifth == BODY || fifth == HEAD || fifth == TAIL);
			tokens[(first*nTotal+second)*vc_size+third].set_used(obj);
                        request_port[first][second][third] = fourth;
                        request_flit_type[first][second][third] = fifth;
                        //fprintf(stderr, " vc request pair (%d,%d,%d,%d,%d)\n", first, second, third, fourth, fifth);
			return &(tokens[(first*nTotal+second)*vc_size+third]);
		}

		void inquire_token(const S& ind, _base_machine_ *obj) {
                  unsigned flit_type = ind.fifth.val();
                  unsigned ins_num = ind.first.val();
                  unsigned interface = ind.fourth.val();
                  unsigned output_vc = ind.third.val();
                  unsigned output_port = ind.second.val();
                  unsigned port_num = saved_input_port[ins_num][output_port][output_vc];
                  unsigned vc_num = saved_input_vc[ins_num][output_port][output_vc];
                  if (interface >0){
                  if (flit_type == 0){

                   credit_score[ind.first.val()][ind.second.val()][ind.third.val()] ++;   
                  }else if(flit_type == TAIL){
                                  /* need to tear down this vc */
                                  vc_status[ins_num][port_num][vc_num]= VC_IDLE;
                                  vc_status_inv[ins_num][output_port][output_vc] = VC_IDLE;
                                
                                  //fprintf(stderr, "vc rid in (%d, %d, %d) for (%d, %d).\n", ins_num, port_num, vc_num, output_port, output_vc);
                                
                                }else{
                                  //std::cout << "vc active " <<std::endl;
                                }   
                  }   
		}


		void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
                  /* find out which index I am in */
                  unsigned offset, ins_num, port_num, vc_num;
                  offset = unsigned (token - tokens);
                  
                  vc_num = offset % vc_size;
                  ins_num = ((offset - vc_num)/vc_size)/nTotal;
                  port_num = ((offset - vc_num)/vc_size)%nTotal;
                  port_list[ins_num][port_num][vc_num] = -1;
                  unsigned flit_type = saved_flit_type[ins_num][port_num][vc_num];
                  unsigned output_port = saved_out_port[ins_num][port_num][vc_num];
                  #if 0
                                if (flit_type == TAIL){
                                  /* need to tear down this vc */
                                  vc_status[ins_num][port_num][vc_num]= VC_IDLE;
                                  vc_status_inv[ins_num][output_port][vc_table[ins_num][port_num][vc_num].second] = VC_IDLE;
                                
                                  //fprintf(stderr, "vc rid in (%d, %d, %d) for (%d, %d).\n", ins_num, port_num, vc_num, output_port, vc_table[ins_num][port_num][vc_num].second);
                                
                                }else{
                                  //std::cout << "vc active " <<std::endl;
                                }
                  #endif
                  if ((token->get_tenant()==obj))
				token->set_unused();
		}

		void discard_token(TOKEN_T *token, _base_machine_ *obj) {
			if (token->get_tenant()==obj)
				token->set_unused();
		}

                T read_token(const S& ind, _base_machine_ *obj) {
                  //std::cout << "read inst=" << (ind.first).val() << " buf_num =" << (ind.second).val()<< " vc_num= "<< (ind.third).val() << " out buf=" << (ind.fourth).val()<<" vctable[1] = "<< vc_table[(ind.first).val()][(ind.second).val()][(ind.third).val()].first << "vctable[2]=" << vc_table[(ind.first).val()][(ind.second).val()][(ind.third).val()].second << std::endl;

                        assert(vc_table[(ind.first).val()][(ind.second).val()][(ind.third).val()].first == (ind.fourth).val());
			return vc_table[(ind.first).val()][(ind.second).val()][(ind.third).val()].second;
		}

		/** Access the value of the token.
		 * This corresponds to val = *token in OSM expression.
		 * @param obj The OSM reading the value.
		 */
		_UINT_T(8) read_token(_base_machine_ *obj, TOKEN_T *token) {
                 
                 int offset, ins_num, port_num,vc_num;
                  offset = unsigned (token - tokens);
                  vc_num = offset % vc_size;
                  ins_num = ((offset - vc_num)/vc_size)/nTotal;
                  port_num = ((offset - vc_num)/vc_size)%nTotal;
		  return vc_table[ins_num][port_num][vc_num].second;
		}

                 #if 0
                 /** Update the value of the token.
		 * This function is called from the OSM. This
		 * corresponds to *m[ind] = val in OSM expression.
		 * To prevent double write in the same cycle, we allocate_temp first.
		 * @param ind The index of the token.
		 * @param val The updated value.
		 * @param obj The OSM doing the update.
		 */
		void write_token(const S& ind, const T& val, _BASE_MACHINE *obj) {
                 assert((ind.first.val() >=0)&&(ind.first.val() < NETWORK_SIZE));
                 assert((ind.second.val() >=0)&&(ind.second.val() < BUFFER_SIZE));
                 assert((ind.third.val() >=0)&&(ind.third.val() < VC_SIZE));
                 if (ind.second.val() !=0){
                          if ((val.val() == 0) ){
                            credit_score[ind.first.val()][ind.second.val()][ind.third.val()] --;
                          }else{
                            credit_score[ind.first.val()][ind.second.val()][ind.third.val()] ++;                       
                          }
                 }
                 assert(credit_score[ind.first.val()][ind.second.val()][ind.third.val()] >=0);
                 assert(credit_score[ind.first.val()][ind.second.val()][ind.third.val()] <= BUFFER_DEPTH);

		}	
  
              #endif

		const std::string& get_name() {
			return name;
		}

		void update_on_clock() {
                  /* calculate the allocated channel table */
                 
                  int arbiter_size_ = nTotal;
                    /* go through all the destination port to make grant list 
                     * description of the algorithm:
                     * 
                     *   0     1    2    3   4        input port
                     * ------------------------
                     * |  2  | 3  | 4  | 1 |  0|      wanted output port
                     *        {c}
                     *  it has a cursor, to do round-robin grant
                     *  if the output port it pointed hasn't been granted, then grant
                     *  if yes, move the cursor to the next one. and remember the 
                     *  last cursor. 
                     */
                    int cursor_port, cursor_vc, output_port;
                    unsigned flit_type;
                    /* init the port_list */
                    for (int j =0; j< count; j++){
                      for (int i =0; i < arbiter_size_; i++){
                        output_score[j][i] = false;
                        input_score[j][i] = false;
                       for (int k =0; k< vc_size; k++){
                         //port_list[j][i][k] = -1;
                      }
                     }
                    }
                    
                    /* allocate a new channel */
                    for (int j =0; j< count; j++){
                     cursor_port = cursor_last_[j];
                     for (int i =0; i < arbiter_size_; i++){
                        cursor_vc = cursor_vc_last_[j][i];
                        for (int k =0; k < vc_size; k++){
                          output_port =  request_port[j][cursor_port][cursor_vc];                                                          
                          flit_type = request_flit_type[j][cursor_port][cursor_vc]; 
                          int in_granting = port_list[j][cursor_port][cursor_vc];                          
                          if ( output_port != -1  && in_granting ==-1){
                            //std::cout << "get_in ins=" << j << "in port="<< cursor_port << " in vc =" <<cursor_vc <<" out port=" << output_port<<std::endl; 
                           /* check if we have this vc already active*/
                            //fprintf(stderr, " vc pair (%d,%d,%d,%d), flit=%d\n",j, cursor_port, cursor_vc,output_port,flit_type);
                           switch (vc_status[j][cursor_port][cursor_vc]){
                             
                            case VC_ACTIVE:
                              if (flit_type == TAIL || flit_type== BODY){
                              
                              //fprintf(stderr, " vc active pair (%d,%d,%d,%d,%d), flit=%d\n",j, cursor_port, cursor_vc,output_port,vc_table[j][cursor_port][cursor_vc].second, flit_type);
                              /* check if the active vc is the one */
                              if (port_list[j][cursor_port][cursor_vc] ==-1 && (!input_score[j][cursor_port]) && (!output_score[j][output_port])){

                                assert (vc_table[j][cursor_port][cursor_vc].first == output_port);
                                int output_vc = vc_table[j][cursor_port][cursor_vc].second;

                                port_list[j][cursor_port][cursor_vc] = output_port;
                                credit_score[j][output_port][output_vc] --;
                                
                                input_score[j][cursor_port] = true;
                                output_score[j][output_port] = true;
                                
                                
                                saved_flit_type[j][cursor_port][cursor_vc] = flit_type;
                                saved_out_port[j][cursor_port][cursor_vc] = output_port;
                                saved_input_port[j][output_port][output_vc] = cursor_port;
                                saved_input_vc[j][output_port][output_vc] = cursor_vc;
                                request_flit_type[j][cursor_port][cursor_vc] = -1;
                                request_port[j][cursor_port][cursor_vc] = -1;
                                   #if 0 
                                  if (flit_type == TAIL){
                                  /* need to tear down this vc */
                                  vc_status[j][cursor_port][cursor_vc] = VC_IDLE;
                                  vc_status_inv[j][output_port][vc_table[j][cursor_port][cursor_vc].second] = VC_IDLE;
                                
                                  std::cout << "vc rid "<< j <<" " << output_port <<" " << cursor_vc <<std::endl;
                                
                                }else{
                                  //std::cout << "vc active " <<std::endl;
                                }
                                #endif
                              }
                              }else{

                              }

                              break;
                            
                           case VC_IDLE:
                             //std::cout << "vc idle " <<std::endl;
                             if (vc_size >1)
                               if (flit_type != HEAD){
                                 std::cout << "vc idle for " <<j <<" empty (" <<cursor_port <<","<< cursor_vc <<" )"<<"flit type "<<flit_type <<std::endl;
                               }
                             assert(flit_type == HEAD);
                            //std::cout << "vc idle " <<port_list[j][cursor_port][cursor_vc] << "input" <<input_score[j][cursor_port] <<" output= "<< output_score[j][output_port]<<std::endl;
                           if (port_list[j][cursor_port][cursor_vc] ==-1 && (!input_score[j][cursor_port]) && (!output_score[j][output_port])){
                            /* found an empty output port, scan for output vc*/
                             int l;
                            for (l=0; l < vc_size; l++){
                              if (vc_status_inv[j][output_port][l] == VC_IDLE && credit_score[j][output_port][l] >= BUFFER_DEPTH) break;
                            }
                            if (l<vc_size){
                            /* we found a empty output vc */
                            port_list[j][cursor_port][cursor_vc] = output_port;
                            credit_score[j][output_port][l]--;
                            saved_flit_type[j][cursor_port][cursor_vc] = flit_type;
                            saved_out_port[j][cursor_port][cursor_vc] = output_port;
                            if (vc_size >1){
                            vc_status_inv[j][output_port][l] = VC_ACTIVE;
                            vc_status[j][cursor_port][cursor_vc] = VC_ACTIVE;
                            }
                            vc_table[j][cursor_port][cursor_vc].first = output_port;
                            vc_table[j][cursor_port][cursor_vc].second = l;
                            input_score[j][cursor_port] = true;
                            output_score[j][output_port] = true;
                            request_port[j][cursor_port][cursor_vc] = -1;
                            request_flit_type[j][cursor_port][cursor_vc] = -1;
                            //std::cout << "assigned inst=" << j << " buf_num =" << cursor_port << " vc_num= "<<cursor_vc << " out buf=" <<output_port<<" out vc = "<< l <<std::endl;
                            }else{
                            /* we run out of empty output vc */
                            // do nothing   
                            }

                           }
                           break;

                         }
                           //switch ends here
                     }
                    cursor_vc = (++cursor_vc) % vc_size;
                     }

                    cursor_port = (++cursor_port) % arbiter_size_;
                   }
                  
                  }

                     /* update cursor */
                  for (int j =0; j< count; j++){
                    cursor_last_[j] = (++cursor_last_[j]) % arbiter_size_;
                  }
                     /* update cursor */
                  for (int j =0; j< count; j++){
                      for (int i =0; i < arbiter_size_; i++){
                       cursor_vc_last_[j][i] = (++cursor_vc_last_[j][i]) % vc_size;
                   }
                  }

	        }

	private:

		std::string name;
                int cursor_last_[count];
                int cursor_vc_last_[count][nTotal];
                /* use to check only one output port is granted */
                bool output_score[count][nTotal];
                /* use to check only one output port is granted */
                bool input_score[count][nTotal];
                /* all the info from the request */
	        int request_port[count][nTotal][vc_size]; 
                int request_flit_type[count][nTotal][vc_size]; 
                int port_list[count][nTotal][vc_size];
                int saved_flit_type[count][nTotal][vc_size];
                int saved_out_port[count][nTotal][vc_size]; 
                int saved_input_port[count][nTotal][vc_size]; 
                int saved_input_vc[count][nTotal][vc_size]; 
                int vc_status[count][nTotal][vc_size];
                int vc_status_inv[count][nTotal][vc_size];
                unsigned credit_score[count][nTotal][vc_size];
                std::pair<int,int>  vc_table[count][nTotal][vc_size];
                std::pair<int,int>  vc_table_inv[count][nTotal][vc_size];
		TOKEN_T *tokens;
	
};

template <class S, class T, int count, int size, int depth, int vc_size>
class ns_vc_buffer_manager_template {

public:

  typedef class _token_<ns_vc_buffer_manager_template<S, T, count, size, depth, vc_size> > TOKEN_T;

  ns_vc_buffer_manager_template(const std::string& name) : name(name) {
    
    real_tokens = new TOKEN_T[count*size*depth*vc_size](this);
    t_index = new (std::pair<int,std::pair<int, int> >)[count*size*depth*vc_size];
    tokens = new std::queue<TOKEN_T *> ***[count];
    token_pool = new std::queue<TOKEN_T *>*[count] ; 
    for (int j = 0; j < count; j++){
            
      tokens[j] = new std::queue<TOKEN_T *> ** [size];
      for (int i=0;i<size;i++){
        tokens[j][i] = new std::queue<TOKEN_T *> *[depth];
         for (int k=0;k<vc_size;k++){
           tokens[j][i][k] = new std::queue<TOKEN_T *>;
              port_counter_write[j][i][k] = 0;
              port_counter_read[j][i][k] = 0;
              vc_status[j][i][k] = VC_IDLE;
              packet_counter[j][i][k] = 0;
              vc_outport[j][i][k] = -1;  
       }
      }  
      token_pool[j] = new std::queue<TOKEN_T *>; 
      for (int i=0;i<(size*depth*vc_size);i++) 
        token_pool[j]->push(real_tokens+(j*size*depth*vc_size)+i);
    
    }
   
  }

  ~ns_vc_buffer_manager_template() {
   for (int j = 0; j < count; j++){
     for (int i=0;i<size;i++){
      for (int k=0;k<vc_size;k++)
       delete tokens[j][i][k];
      delete [] tokens[j][i];
     }
    delete [] tokens[j];
    delete token_pool[j];
    //delete t_index[j];
   }
    delete [] tokens;
    delete [] token_pool;
    delete [] real_tokens;
    delete t_index;
  
  }

  bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
    if((tokens[(ind.first).val()][(ind.second).val()][(ind.third).val()]->size() >= depth )||(port_counter_write[(ind.first).val()][(ind.second).val()][(ind.third).val()]== 1)){ 
      return false;
   } else{
      // check the flit type
       switch (vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()]){
                             
                            case VC_ACTIVE:
                              switch ((ind.fifth).val()){
                                case BODY:
                                  if((packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]<(PACKET_SIZE - 2))&&((vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()] ==(ind.fourth).val()) || (ind.second).val() == 0)){
                                    //fprintf(stderr, " body recv: previous=%d,(%d,%d,%d,%d,%d) with counter = %d\n",vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()],(ind.first).val(),(ind.second).val(),(ind.third).val(),(ind.fourth).val(),(ind.fifth).val(), (packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]));
                                    return true;}
                                  else{
                                    //fprintf(stderr, " fail in alloc: previous=%d,(%d,%d,%d,%d,%d)\n",vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()],(ind.first).val(),(ind.second).val(),(ind.third).val(),(ind.fourth).val(),(ind.fifth).val());
                                    return false;
                                  }
                                case TAIL:
                                  // see if we counted enough body
                                  if (packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]==(PACKET_SIZE - 2)&&((vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()] ==(ind.fourth).val())||(ind.second).val() == 0)) {
                                    //fprintf(stderr, "active got tail with (%d,%d,%d,%d,%d) with counter = %d\n",(ind.first).val(),(ind.second).val(),(ind.third).val(),(ind.fourth).val(),(ind.fifth).val(), (packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]));
                                    return true;
                                  }else{
                                    //fprintf(stderr, "fail got tail with (%d,%d,%d,%d,%d) with counter = %d\n",(ind.first).val(),(ind.second).val(),(ind.third).val(),(ind.fourth).val(),(ind.fifth).val(), (packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]));
                                    return false;
                                  }
                                case HEAD:
                                     //fprintf(stderr, "index= %d, %d, %d, %d\n", (ind.first).val(), (ind.second).val(), (ind.third).val(),(ind.fifth).val()); 
                                     //exit(0);
                                    return false;
                               }
                              break;                              
                            case VC_IDLE:
                              if ((ind.fifth).val()==HEAD || vc_size ==1){
                                return true;
                               }else{
                                 return false;
                                  }
                              break;
                            default:
                              fprintf(stderr, " unknown vc_status in token_free_for alloc.\n");
       }
      
      
    }
  }

bool token_free(const S& ind, _base_machine_ *obj) {
    if((tokens[(ind.first).val()][(ind.second).val()][(ind.third).val()]->size() >(depth-PACKET_SIZE)) ||(port_counter_write[(ind.first).val()][(ind.second).val()][(ind.third).val()]== 1)){ 
      return false;
   } else if (vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()]==VC_ACTIVE && ((ind.fifth).val() == HEAD)){
  return false;
  }else{
      return true;
  }
}

  bool token_free_for_inquire(const S& ind,_base_machine_ *obj) {
    
    switch ((ind.fifth).val()){
      case 0:
        // check if we are at the head
        
        if (obj==(tokens[((ind.first).val())][(ind.second).val()][(ind.third).val()]->front())->get_tenant()){
          //fprintf(stderr, "inquire: at the front\n");
          return true;
        }else{
          return false;
        }
          
        break;
      case HEAD:
      case BODY:
      case TAIL:
        return token_free(ind,obj);
        break;
      default:
        fprintf(stderr, "error: unknown flit_type for inquire.\n");
        return false;
    }
     
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    /* if the token to be reclaimed is the same as the 
     * token at the front of the buffer 
     */
    unsigned offset;
    std::pair<int,std::pair<int,int> > my_index;
   
    offset = unsigned (token - real_tokens);
    my_index = t_index[offset];
     if (port_counter_read[my_index.first][(my_index.second).first][(my_index.second).second] == 0){
    if (token == tokens[my_index.first][(my_index.second).first][(my_index.second).second]->front()){ 
      //fprintf(stderr, "token good to reclaim,(%d, %d, %d) \n", my_index.first, (my_index.second).first, (my_index.second).second);
      return true;
   }else{
     return false; 
   }
  }else{
     return false; 
   }
  }

  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {//

    TOKEN_T * t;
    int offset;
    
    t = token_pool[(ind.first).val()]->front();
	//t->set_used(obj);
    offset = t - real_tokens;
    t_index[offset].first = (ind.first).val();
    (t_index[offset].second).first = (ind.second).val();
    (t_index[offset].second).second = (ind.third).val();
    tokens[(ind.first).val()][(ind.second).val()][(ind.third).val()]->push(t);
    port_counter_write[(ind.first).val()][(ind.second).val()][(ind.third).val()] = 1;
    token_pool[(ind.first).val()]->pop();
    // check the flit type
    if (vc_size > 1){
       switch ((ind.fifth).val()){
                           case HEAD:
                             if (vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()] != VC_IDLE){
                               //fprintf(stderr, "index= %d, %d, %d, %d\n", (ind.first).val(), (ind.second).val(), (ind.third).val(),(ind.fifth).val()); 
                               //fprintf(stderr, "status = %d\n", vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()]);
                             }
                             assert(vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()] == VC_IDLE);
                             
                             vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()] = VC_ACTIVE;
                             packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()] = 0;
                             vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()] = (ind.fourth).val();
                             break;

                           case BODY:
                             //fprintf(stderr, " in allocate got body for (%d, %d, %d) with counter = %d, vc_out = %d, request vc = %d\n",(ind.first).val(),(ind.second).val(),(ind.third).val(),  (packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]), vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()], (ind.fourth).val());
                             assert(vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()] == VC_ACTIVE);
                             assert(vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()] ==(ind.fourth).val());
                             packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()] ++; 
                             
                             
                             break;                            
                           case TAIL:
                             assert(vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()] == VC_ACTIVE);
                             vc_status[(ind.first).val()][(ind.second).val()][(ind.third).val()] = VC_IDLE;
                             vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()] = -1;
                              packet_counter[(ind.first).val()][(ind.second).val()][(ind.third).val()]=0; 
                             break;  
                           default:
                              fprintf(stderr, " unknown flit type\n");
       }
    }else{
      vc_outport[(ind.first).val()][(ind.second).val()][(ind.third).val()] = (ind.fourth).val();
    }
    t->set_used(obj);
    return t;
 
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
   // return (tokens[((ind.first).val())][(ind.second).val()][(ind.third).val()]->front());	
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    unsigned offset;
    std::pair<int,std::pair<int,int> > my_index;
    //token->set_unused();
    // this seems to unuseful code, will clean it later
    offset = unsigned (token - real_tokens);
    my_index.first = t_index[offset].first;
    (my_index.second).first = (t_index[offset].second).first;
    (my_index.second).second = (t_index[offset].second).second;
    tokens[my_index.first][(my_index.second).first][(my_index.second).second] -> pop();
    token_pool[my_index.first]->push(token);      
    port_counter_read[my_index.first][(my_index.second).first][(my_index.second).second]= 1;
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    reclaim_token(token,obj);
    
  
  }
 
  T read_token (_base_machine_ *obj, TOKEN_T *token){
    unsigned offset;
    std::pair<int,std::pair<int,int> > my_index;
   
    offset = unsigned (token - real_tokens);
    my_index = t_index[offset];
  
    if (token == tokens[my_index.first][(my_index.second).first][(my_index.second).second]->front()){ 
      return 1;
   }else{
      return 0;
   }
  
 }
  
  const std::string& get_name() {return name;}

  void update_on_clock(){

   for (int j = 0; j < count; j++){
    for (int i=0;i<size;i++){
      for (int k=0;k<vc_size;k++){
      port_counter_write[j][i][k] = 0;
      port_counter_read[j][i][k] = 0; 
    }
   }
  }
  }
   
private:

  std::string name;
  TOKEN_T *real_tokens;
  std::pair<int,std::pair<int,int> >* t_index;
  std::queue<TOKEN_T*> ****tokens;
  std::queue<TOKEN_T *> **token_pool;
  int port_counter_read[count][size][vc_size];
  int port_counter_write[count][size][vc_size];
  int vc_status[count][size][vc_size];
  int packet_counter[count][size][vc_size];
  int vc_outport[count][size][vc_size];
};


template<class S, class T, int range, int prob>
class ns_pipe_fetch_manager {//

public:

  typedef class _token_<ns_pipe_fetch_manager<S, T, range, prob> > TOKEN_T;

  ns_pipe_fetch_manager(const std::string& name) :
    name(name){
    real_tokens = new TOKEN_T[MAX_TOKEN_COUNT](this);
    real_values = new int [MAX_TOKEN_COUNT];
    token_pool = new std::queue<TOKEN_T *>*[range];
    token_queue = new std::queue<TOKEN_T *>;
    for (int j = 0; j < range; j++){

      token_pool[j] = new std::queue<TOKEN_T *>; 
    } 
    for (int i=0;i<(MAX_TOKEN_COUNT);i++){
        token_queue->push(real_tokens+i);
    
    }
  }

  ~ns_pipe_fetch_manager() {
   for (int j = 0; j < range; j++){
    delete token_pool[j];
   }
   delete token_queue;
   delete [] token_pool;
   delete [] real_values;
   delete [] real_tokens;
  
 }

  bool token_free_for_alloc(const S& ind,  _base_machine_ *obj) {
    // check if we have stuff in this queue 
    if(token_pool[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  
  }

  bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
    if(tokens[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    return true;
  }
        
  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    
    TOKEN_T * t;
    int offset;
    t = token_pool[ind.val()]->front();
    offset = t - real_tokens;
    real_values[offset] = ind.val();
    return t;
  }

  TOKEN_T *allocate_token_temp(const S& ind, _base_machine_ *obj) {
    TOKEN_T * t;
    t = token_pool[ind.val()]->front();
    return t;
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  T read_token (_base_machine_ *obj, TOKEN_T *token)
		{
                  return 1;
                }

  const std::string& get_name() {return name;}
  
  void update_on_clock(){
    // for each instance, see if we have the probablity to generate a packet 
    unsigned int r1, r2;
    r1 = ((unsigned)-1)/100*prob;
    for (int j = 0; j < range; j++){

	r2 = rand();
        if (r1 > r2){
          // put a token in the injection pipe
          if (!token_queue->empty()){
            token_pool[j]->push(token_queue->front());
            token_queue->pop();
          }else{
            fprintf(stderr, " run out of queue space\n");
          }
        }
   }
  }

private:

  std::string name;
  TOKEN_T *real_tokens;
  int *real_values;
  std::queue<TOKEN_T*> **token_pool;
  std::queue<TOKEN_T*> *token_queue;
};

simulator::arm_simulator *new_arm_simulator(bool);
void init_arm_simulator(simulator::arm_simulator *sim);
void run_arm_simulator(simulator::arm_simulator *sim);
void unregister_device(simulator::arm_simulator *pe_sim, unsigned address);
void register_if_device(simulator::arm_simulator *pe_sim, unsigned address,
	emulator::arm_if_device *pe_if_dev);
void register_id_device(simulator::arm_simulator *pe_sim, unsigned address,
	emulator::arm_id_device *pe_id_dev);

template<class S, class T, int range>
class ns_arm_fetch_manager {//

public:

  typedef class _token_<ns_arm_fetch_manager<S, T, range> > TOKEN_T;
 

  ns_arm_fetch_manager(const std::string& name) :
    name(name){
    real_tokens = new TOKEN_T[MAX_TOKEN_COUNT](this);
    real_values = new T [MAX_TOKEN_COUNT];
    token_pool = new std::queue<TOKEN_T *>*[range];
    token_queue = new std::queue<TOKEN_T *>;
    for (int j = 0; j < range; j++){

      token_pool[j] = new std::queue<TOKEN_T *>; 
    } 
    for (int i=0;i<(MAX_TOKEN_COUNT);i++){
        token_queue->push(real_tokens+i);
    
    }
    /* init the pe simulator */
	pe_sim = new_arm_simulator(false);
        init_arm_simulator(pe_sim);
    /* init the pe interface device */
    pe_if_dev = new emulator::arm_if_device();
    pe_id_dev = new emulator::arm_id_device();
    dev_if_registered = false;
    dev_id_registered = false;
    register_if_device(pe_sim,1, pe_if_dev);
    register_id_device(pe_sim,0, pe_id_dev);
  }

  ~ns_arm_fetch_manager() {
   for (int j = 0; j < range; j++){
    delete token_pool[j];
   }
   delete token_queue;
   delete [] token_pool;
   delete [] real_values;
   delete [] real_tokens;
   //unregister_device(pe_sim, address);
   //unregister_device(pe_sim, address);
   delete pe_if_dev;
   delete pe_id_dev;
 }

  bool token_free_for_alloc(const S& ind,  _base_machine_ *obj) {
   // check if we have stuff in this queue 
    
  return true;
  }

  void set_address(unsigned a){
        pe_id_dev->set_address(a);
        address = a;   
        //fprintf(stderr, "address = %d\n", address);   
  }

  bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
    
      if(token_pool[0]->empty()){
        return false;
      } else{
        return true;
      }
   
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    return true;
  }
        
  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    
    TOKEN_T * t;
    int offset;
    t = token_pool[0]->front();
    return t;
  }

  TOKEN_T *allocate_token_temp(const S& ind, _base_machine_ *obj) {
    TOKEN_T * t;
    t = token_pool[0]->front();
    return t;
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[0]->pop();
    //fprintf(stderr, "in %d , token reclaimed\n", address);
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[0]->pop();
  }

  T read_token (_base_machine_ *obj, TOKEN_T *token)
		{
                  unsigned offset = unsigned (token - real_tokens);
                  T value = real_values[offset];
                  //fprintf(stderr, "read token address= %d with (%d,%d)\n", address, value.first.val(), value.second.val());
                  return value;
                }

  T read_token(const S& ind, _base_machine_ *obj) {
                     TOKEN_T * t;
                     int offset;
                     t = token_pool[0]->front();
                     offset = unsigned (t - real_tokens);
                     T value = real_values[offset];
                     //fprintf(stderr, "read token address= %d with (%d,%d)\n", address, value.first.val(), value.second.val()); 
                     token_queue->push(t);
                     token_pool[0]->pop();
                     return real_values[offset]; 
                     
                }


  
               /** Update the value of the token.
		 * This function is called from the OSM. This
		 * corresponds to *m[ind] = val in OSM expression.
		 * To prevent double write in the same cycle, we allocate_temp first.
		 * @param ind The index of the token.
		 * @param val The updated value.
		 * @param obj The OSM doing the update.
		 */
		void write_token(const S& ind, const T& val, _BASE_MACHINE *obj) {
                  pe_if_dev->put_input_queue(emulator::ns_data(val.first.val(), val.second.val()));
                  //fprintf(stderr, "write token address= %d with (%d,%d)\n", address, val.first.val(), val.second.val());
		}											
  const std::string& get_name() {return name;}
  
  void update_on_clock(){
    // for each instance, see if we have the probablity to generate a packet 
        if (pe_if_dev->get_output_queue_status()){
          // put a token in the injection pipe
          if (!token_queue->empty()){
            
            TOKEN_T* token = token_queue->front();
            unsigned offset = token - real_tokens;
            token_pool[0]->push(token);
            token_queue->pop();
            emulator::ns_data data;
            pe_if_dev->get_output_queue(data);
            unsigned dest_addr = data.first;
            unsigned real_data = data.second;  
            T value = T (dest_addr, real_data);
            real_values[offset] = value;
            //fprintf(stderr, "got in address = %d, dest = %d, data= %d\n", address, data.first, data.second);
          }else{
            fprintf(stderr, " run out of queue space\n");
          }
        }
        run_arm_simulator(pe_sim);

  }

private:

  std::string name;
  TOKEN_T *real_tokens;
  T* real_values;
  std::queue<TOKEN_T*> **token_pool;
  std::queue<T> *value_pool;
  std::queue<TOKEN_T*> *token_queue;
  simulator::arm_simulator *pe_sim;
  emulator::arm_if_device *pe_if_dev;
  emulator::arm_id_device *pe_id_dev;
  bool dev_if_registered;
  bool dev_id_registered;
  unsigned address;
};


template<class S, class T, int range, int prob>
class ns_bus_fetch_manager {//

public:

  typedef class _token_<ns_bus_fetch_manager<S, T, range, prob> > TOKEN_T;

  ns_bus_fetch_manager(const std::string& name) :
    name(name){
    real_tokens = new TOKEN_T[MAX_TOKEN_COUNT](this);
    real_values = new int [MAX_TOKEN_COUNT];
    token_pool = new std::queue<TOKEN_T *>*[range];
    token_queue = new std::queue<TOKEN_T *>;
    for (int j = 0; j < range; j++){

      token_pool[j] = new std::queue<TOKEN_T *>; 
    } 
    for (int i=0;i<(MAX_TOKEN_COUNT);i++){
        token_queue->push(real_tokens+i);
    
    }
  }

  ~ns_bus_fetch_manager() {
   for (int j = 0; j < range; j++){
    delete token_pool[j];
   }
   delete token_queue;
   delete [] token_pool;
   delete [] real_values;
   delete [] real_tokens;
  
 }

  bool token_free_for_alloc(const S& ind,  _base_machine_ *obj) {
    // check if we have stuff in this queue 
    if(token_pool[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  
  }

  bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
    if(token_pool[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    return true;
  }
        
  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    
    TOKEN_T * t;
    int offset;
    t = token_pool[ind.val()]->front();
    offset = t - real_tokens;
    real_values[offset] = ind.val();
    return t;
  }

  TOKEN_T *allocate_token_temp(const S& ind, _base_machine_ *obj) {
    TOKEN_T * t;
    t = token_pool[ind.val()]->front();
    return t;
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  T read_token (_base_machine_ *obj, TOKEN_T *token)
		{
                  return 1;
                }

  const std::string& get_name() {return name;}
  
  void update_on_clock(){
    // for each instance, see if we have the probablity to generate a packet 
    unsigned int r1, r2;
    r1 = ((unsigned)-1)/100*prob;
    for (int j = 0; j < range; j++){

	r2 = rand();
        if (r1 > r2){
          // put a token in the injection pipe
          if (!token_queue->empty()){
            token_pool[j]->push(token_queue->front());
            token_queue->pop();
          }else{
            fprintf(stderr, " run out of queue space\n");
          }
        }

   }
  }
private:

  std::string name;
  TOKEN_T *real_tokens;
  int *real_values;
  T * stored_values;
  std::queue<TOKEN_T*> **token_pool;
  std::queue<TOKEN_T*> *token_queue;
  int count;
};

template<class S, class T, int range, int read_write_mode>
class ns_cc_bus_slave_response_manager {//

public:

  typedef class _token_<ns_cc_bus_slave_response_manager<S, T, range, read_write_mode> > TOKEN_T;
 
  ns_cc_bus_slave_response_manager(const std::string& name) :
    name(name){

    real_tokens = new TOKEN_T[MAX_TOKEN_COUNT](this);
    real_values = new int [MAX_TOKEN_COUNT];
    stored_values= new T [MAX_TOKEN_COUNT];
    token_pool = new std::queue<TOKEN_T *>*[range];    
    token_queue = new std::queue<TOKEN_T *>;
    for (int j = 0; j < range; j++){
      token_pool[j] = new std::queue<TOKEN_T *>;    
    } 
    for (int i=0;i<(MAX_TOKEN_COUNT);i++){
        token_queue->push(real_tokens+i);    
    }
    /* reading the reponses from the slave bfl files */
     for (int ins=0; ins<range; ins++){
        char bfl_name[20];
        sprintf(bfl_name, "%s_%d.bfl", BFL_SLAVE_PREFIX, ins);
        
        FILE * pFile;
        pFile = fopen (bfl_name,"rt");
        if (pFile==NULL){               
          cerr << "Cannot open input file " << bfl_name << endl;
          exit(-1);
        }else{
          cerr << "Reading SLAVE BFL file from " <<bfl_name <<endl;
        }
        char my_line [100]={0};;
        char my_newline [100]={0};
        int n =0;
        // process the file and fill the queue.
        while (!feof(pFile)) {
          char *x = fgets (my_line, 100, pFile);
          if (x== NULL) break;
          //see if the first several word is read or write 
          int ln=strlen(my_line);
          //cerr<< "line: " << my_line << endl;
          int read_write = -1;



          if (read_write_mode == GENERIC_READ &&strncmp (my_line,"read_response(",14) == 0){
            // it is read
            read_write = GENERIC_READ;
            strncpy(my_newline, my_line+14, ln-16);
                
          }else if (read_write_mode == GENERIC_WRITE && strncmp (my_line,"write_response(",15) == 0){
            // it is write
            read_write = GENERIC_WRITE;
            strncpy(my_newline, my_line+15, ln-17);
          }else{
            if ( !(strncmp (my_line,"read_response(",14) == 0 ||strncmp (my_line,"write_response(",15) == 0)){
            cerr << "In file " << bfl_name <<" line " << n << " neither read_response or write_response"<< endl;
            }
            continue;
          } 
          
          char *pch = strtok (my_newline,",");
          char aack_line[20];
          char dack_line[20];
          char rearb_line[20];
          char burst_line[20];
          char wait_line[20];
          int aack_num, dack_num, rearb_num, burst_num, wait_num;
          aack_num = dack_num = rearb_num = burst_num = wait_num = 0;
          while (pch != NULL)
            {
              if (strncmp (pch,"aack_delay=",11)==0){
                strcpy(aack_line, pch+11);
                aack_num = strtol(aack_line, (char **) NULL, 10);
              }else if (strncmp (pch,"dack_delay=",11)==0){
                strcpy(dack_line, pch+11);
                dack_num = strtol(dack_line, (char **) NULL, 10);
              }else if (strncmp (pch,"rearb_delay=",12)==0){
                strcpy(rearb_line, pch+12);
                rearb_num = strtol(rearb_line, (char **) NULL, 10);
              }else if (strncmp (pch,"wait_delay=",11)==0){
                strcpy(wait_line, pch+11);
                wait_num = strtol(wait_line, (char **) NULL, 10);
              }else if (strncmp (pch,"burst_term=",11)==0){
                strcpy(burst_line, pch+11);
                burst_num = strtol(burst_line, (char **) NULL, 10);
              }
              pch = strtok (NULL, " ,");
            }
         
          cerr << "Entry: aack_delay= " << aack_num << " dack_delay= " <<dack_num;
          cerr << " rearb_delay= " << rearb_num << " wait_delay = " ;
          cerr << wait_num << " burst_term = " << burst_num << endl;
        
          // we have all the tuple data set, fill the token
       
             TOKEN_T *token;
             int offset;
             token = token_queue->front();
             offset = token - real_tokens;                         
             T value = T (aack_num, rearb_num, dack_num, wait_num, burst_num);
             stored_values[offset] = value;
             // ins is the local address  
             token_pool[ins]->push(token);
             token_queue->pop();        
        }
        fclose(pFile);
      }

  }

 ~ns_cc_bus_slave_response_manager() {
   for (int j = 0; j < range; j++){
    delete token_pool[j];   
   }

   delete token_queue;
   delete [] token_pool;
   delete [] real_values;
   delete [] real_tokens;
   delete [] stored_values;  
 }

  bool token_free_for_alloc(const S& ind,  _base_machine_ *obj) {
    // check if we have stuff in this queue 
    if(token_pool[ind.val()]->empty()){
       return false;
    } else{
      return true;
    } 
  }

  bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
    if(token_pool[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    return true;
  }
        
  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    
    TOKEN_T * t;
    int offset;
    t = token_pool[ind.val()]->front();
    offset = t - real_tokens;
    real_values[offset] = ind.val();
    return t;
  }

  TOKEN_T *allocate_token_temp(const S& ind, _base_machine_ *obj) {
    TOKEN_T * t;
    t = token_pool[ind.val()]->front();
    return t;
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  T read_token (_base_machine_ *obj, TOKEN_T *token)
 {
                 unsigned offset = unsigned (token - real_tokens);
                  T value = stored_values[offset];
                  return value;
                
 }

  const std::string& get_name() {return name;}
  
  void update_on_clock(){
    TOKEN_T *token;
    int offset;
 }


private:

  std::string name;
  TOKEN_T *real_tokens;
  int *real_values;
  T * stored_values;
  std::queue<TOKEN_T*> **token_pool;
  std::queue<TOKEN_T*> *token_queue;
  int count;
};


template<class S, class T, int range, int prob, int config>
class ns_cc_bus_fetch_manager {//

public:

  typedef class _token_<ns_cc_bus_fetch_manager<S, T, range, prob, config> > TOKEN_T;
  typedef class _QUINTUPLE_T(_UINT_T(8),_UINT_T(8), _UINT_T(8),_UINT_T(8), _UINT_T(8)) MY_QUIN;
  ns_cc_bus_fetch_manager(const std::string& name) :
    name(name){

    real_tokens = new TOKEN_T[MAX_TOKEN_COUNT](this);
    real_values = new int [MAX_TOKEN_COUNT];
    stored_values= new T [MAX_TOKEN_COUNT];
    token_pool = new std::queue<TOKEN_T *>*[range];
    trans_queue = new std::queue<MY_QUIN *>*[range];
    token_queue = new std::queue<TOKEN_T *>;
    for (int j = 0; j < range; j++){

      token_pool[j] = new std::queue<TOKEN_T *>; 
      trans_queue[j] = new std::queue<MY_QUIN *>;
    } 
    for (int i=0;i<(MAX_TOKEN_COUNT);i++){
        token_queue->push(real_tokens+i);
    
    }
    count = 0;
    // this fetch manager operates in two modes.
    // if config == 0, it process a BFL file to get all bus test cases
    // if config == TEST_CASE_NAME, it use internal built-in test cases

    if (!config){
      // open the BFL filename
      for (int ins=0; ins<range; ins++){
        char bfl_name[20];
        sprintf(bfl_name, "%s_%d.bfl", BFL_MASTER_PREFIX, ins);
        
        FILE * pFile;
        pFile = fopen (bfl_name,"rt");
        if (pFile==NULL){               
          cerr << "Cannot open input file " << bfl_name << endl;
          exit(-1);
        }else{
          cerr << "Reading MASTER BFL file from " <<bfl_name <<endl;
        }
        char my_line [100]={0};
        char my_newline [100]={0};
        int n =0;
        // process the file and fill the queue.
        while (!feof(pFile)) {
          char *x = fgets (my_line, 100, pFile);
          if (x== NULL) break;
          //see if the first several word is read or write 
          int ln=strlen(my_line);
          //cerr<< "line: " << my_line << endl;
          int read_write = -1;
          if (strncmp (my_line,"read(",5) == 0){
            // it is read
            read_write = GENERIC_READ;
            strncpy(my_newline, my_line+5, ln-7);
          
          }else if (strncmp (my_line,"write(",6) == 0){
            // it is write
            read_write = GENERIC_WRITE;
            strncpy(my_newline, my_line+6, ln-8);
          }else{
            cerr << "In file " << bfl_name <<" line " << n << " neither read or write"<< endl;
            continue;
          }
          char *pch;
          pch = strtok (my_newline,",");
          char addr_line[20];
          char size_line[20];
          char req_delay_line[20];
          char be_line[20];
          char burst_count_line[20];
          char lock_line[20];
          char unlock_line[20];
          char abort_line[20];
          char burst_deassert_line[20];
          int addr_num, size_num, req_delay_num, burst_count_num, be_num;
          int abort_num, unlock_num, lock_num, burst_deassert_num;
          addr_num = size_num = req_delay_num = burst_count_num = be_num = 0;
          abort_num = unlock_num = lock_num = burst_deassert_num = 0;
          while (pch != NULL)
            {
              if (strncmp (pch,"addr=",5)==0){
                strcpy(addr_line, pch+5);
                addr_num = strtol(addr_line, (char **) NULL, 16);
              }else if (strncmp (pch,"size=",5)==0){
                strcpy(size_line, pch+5);
                size_num = strtol(size_line, (char **) NULL, 2);
              }else if (strncmp (pch,"req_delay=",10)==0){
                strcpy(req_delay_line, pch+10);
                req_delay_num = strtol(req_delay_line, (char **) NULL, 10);
              }else if (strncmp (pch,"burst_count=",12)==0){
                strcpy(burst_count_line, pch+12);
                burst_count_num = strtol(burst_count_line, (char **) NULL, 10);
              }else if (strncmp (pch,"be=",3)==0){
                //strncpy(be_line, pch+3, strlen(pch)-8);
                strcpy(be_line, pch+3);
                be_num = strtol(be_line, (char **) NULL, 2);
              }else if (strncmp (pch,"lock=",5)==0){
                strcpy(lock_line, pch+5);
                lock_num = strtol(lock_line, (char **) NULL, 10);
              }else if (strncmp (pch,"unlock=",7)==0){
                strcpy(unlock_line, pch+7);
                unlock_num = strtol(unlock_line, (char **) NULL, 10);
              }else if (strncmp (pch,"abort_delay=",12)==0){
                strcpy(abort_line, pch+12);
                abort_num = strtol(abort_line, (char **) NULL, 10);
              }else if (strncmp (pch,"burst_deassert_delay=",21)==0){
                strcpy(burst_deassert_line, pch+21);
                burst_deassert_num = strtol(burst_deassert_line, (char **) NULL, 10);
              }
              pch = strtok (NULL, " ,");
            }
          cerr << req_delay_line <<endl;
          cerr << "Entry: addr= " << addr_num << " be = " << be_num;
          cerr << " size = " <<size_num;
          cerr << " burst_count= " << burst_count_num << " req_delay = " ;
          cerr << req_delay_num << " lock = " << lock_num << " unlock = " ;
          cerr << unlock_num << " abort_delay = " << abort_num; 
          cerr << " burst_de_assertnum = " << burst_deassert_num  <<endl;
          int trans_type = read_write;
          int trans_size = be_num;
          
          if (read_write == GENERIC_READ && burst_count_num > 1){
               trans_type = BUS_BURST_READ;
               trans_size = burst_count_num;
             }else if (read_write == GENERIC_WRITE && burst_count_num > 1){
               trans_type = BUS_BURST_WRITE;
               trans_size= burst_count_num;
             }else if (burst_count_num <=1){
               /* not burst read, single beat, check size */
               if (size_num == 0){ 
                 trans_size = 1;
               }else{
                 switch (size_num){
                 
                 case 1: trans_size = 4; break;
                 case 2: trans_size = 8; break;
                 case 3: trans_size = 16; break;
                 case 8:
                 case 9:
                 case 10:
                 case 11:
                 case 12:
                 case 13: trans_size = burst_deassert_num; break;
                 default: cerr << "Error: in single-beat transfer mode: size = " << size_num <<endl;
                }
                if (read_write == GENERIC_WRITE ){
                 trans_type = BUS_BURST_WRITE;
               }else{
                 trans_type = BUS_BURST_READ;
                }
               }
               }else if (burst_count_num == 0 && be_num == 0 && (size_num>=7 || size_num <=17)){
               // in the slave termination mode 
               if (read_write == GENERIC_WRITE ){
                 trans_type = BUS_BURST_WRITE;
               }else{
                 trans_type = BUS_BURST_READ;
               }
               // if burst_deassert_num == 0, slave termination mode
               // if burst_deassert_num > 0, master termination mode
               // note: slight difference: since fixed-length is implemented
               // as the same as the non-fixed-length, only add an extra 
               // cycle occupied on the beat since the bus won't be availabe
               // until beat+1 cycles later.
               if (burst_deassert_num > 0){
                 trans_size= burst_deassert_num + 1;
               }else{
                 trans_size = 0;
               }
             }
          if ((trans_type == BUS_BURST_READ || trans_type == GENERIC_READ) && lock_num){
            trans_type = BUS_LOCK_READ;
          }else if ((trans_type == BUS_BURST_READ || trans_type == GENERIC_READ) && unlock_num){
            trans_type = BUS_UNLOCK_READ;
          }else if ((trans_type == BUS_BURST_WRITE || trans_type == GENERIC_WRITE) && lock_num){
            trans_type = BUS_LOCK_WRITE;
          }else if ((trans_type == BUS_BURST_WRITE || trans_type == GENERIC_WRITE) && unlock_num){
            trans_type = BUS_UNLOCK_WRITE;
          }
           assert(addr_num >= 0 && addr_num < range);
           assert(trans_size >= 0 && trans_size < 16);
           assert(abort_num >=0);
          // we have all the tuple data set, fill the token
          if (req_delay_num == 0){
            // fill in the token queue right now            
             TOKEN_T *token;
             int offset;
             token = token_queue->front();
             offset = token - real_tokens;                         
             T value = T (addr_num, trans_type, trans_size, abort_num);
             cerr << "manager: addr = " << addr_num << " trans_type = " << trans_type << " trans_size= " << trans_size <<endl;
             stored_values[offset] = value;
             // ins is the local address  
             token_pool[ins]->push(token);
             token_queue->pop();
          }else{
            // fill in the transaction queue for later
            // format quaduple(addr, trans_type, trans_size, trans_time);
            MY_QUIN *trans = new MY_QUIN(addr_num,trans_type,trans_size,req_delay_num, abort_num);
            trans_queue[ins]->push(trans);            
          }
        }
        fclose(pFile);
      }
    }
  }

  ~ns_cc_bus_fetch_manager() {
   for (int j = 0; j < range; j++){
    delete token_pool[j];
    delete trans_queue[j];
   }
   delete token_queue;
   delete [] token_pool;
   delete [] trans_queue;
   delete [] real_values;
   delete [] real_tokens;
   delete [] stored_values;  
 }

  bool token_free_for_alloc(const S& ind,  _base_machine_ *obj) {
    // check if we have stuff in this queue 
    if(token_pool[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  
  }

  bool token_free_for_inquire(const S& ind, _base_machine_ *obj) {
    if(token_pool[ind.val()]->empty()){
      return false;
    } else{
      return true;
    }
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    return true;
  }
        
  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    
    TOKEN_T * t;
    int offset;
    t = token_pool[ind.val()]->front();
    offset = t - real_tokens;
    real_values[offset] = ind.val();
    return t;
  }

  TOKEN_T *allocate_token_temp(const S& ind, _base_machine_ *obj) {
    TOKEN_T * t;
    t = token_pool[ind.val()]->front();
    return t;
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    int ind,offset;
    offset = token - real_tokens;
    ind = real_values[offset];
    if (token->get_tenant()==obj)
      token->set_unused();
    token_queue->push(token);
    token_pool[ind]->pop();
  }

  T read_token (_base_machine_ *obj, TOKEN_T *token)
 {
                 unsigned offset = unsigned (token - real_tokens);
                  T value = stored_values[offset];
                  return value;
                
 }

  const std::string& get_name() {return name;}
  
  void update_on_clock(){
    TOKEN_T *token;
    int offset;
    count ++;
  
    switch(config){
      case 0:
        // peek the front of the queue for each trans queue
        for (int ins = 0; ins <range; ins++){
          if (!trans_queue[ins]->empty()){
          MY_QUIN * trans = trans_queue[ins]->front();
          int delay = trans->fourth.val();
          if (count >= delay){
            // should be activated
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (trans->first.val(), trans->second.val(), trans->third.val(), trans->fifth.val()); 
            stored_values[offset] = value; 
            token_pool[ins]->push(token);
            token_queue->pop();
            trans_queue[ins]->pop();
            delete trans;
          }
          }          
        }
        break;
      case 521:
      case 522: 
        assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0 ); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        if (count == 3) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 2, 0); // 1-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[2]->push(token);
            token_queue->pop();
        }
        if (count == 17) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0); // 1-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        break;
    case 523:
    case 524:
       assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        if (count == 3) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 2, 0); // 1-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[2]->push(token);
            token_queue->pop();
        }
        if (count == 17) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0); // 1-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        break;
       
    case 525:
         assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[2]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[3]->push(token);
            token_queue->pop();
        }  
        break;
     case 526:
         assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_BURST_READ, 4, 0); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_BURST_READ, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        break;
       case 527:
         assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_BURST_WRITE, 4, 0); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_BURST_WRITE, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        break;
       case 513:
          assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0 ); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4, 0); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        if (count == 3) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0 ); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 4) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0 ); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        break;
       case 518:
       assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_LOCK_READ, 4, 0); // 4-word-line lock read from slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
      
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_UNLOCK_WRITE, 4, 0 ); // 4-word-line write to slave 3
            stored_values[offset] = value; 
            token_pool[0]->push(token);
            token_queue->pop();
        }
        if (count == 2) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_READ, 4 ,0  ); // 4-word-line read from slave 3
            stored_values[offset] = value; 
            token_pool[2]->push(token);
            token_queue->pop();
        }
        break;
        case 519:
       assert(range >=3);
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_WRITE, 4, 0 ); // 4-word-line lock read from slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        break;
       case 520:
       assert(range >=3);
        
        if (count == 1) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_LOCK_READ, 4, 0); // 4-word-line lock read from slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        
        if (count == 8) {
            token = token_queue->front();
            offset = token - real_tokens;
            T value = T (3, BUS_UNLOCK_READ, 1, 0); // 4-word-line lock read from slave 3
            stored_values[offset] = value; 
            token_pool[1]->push(token);
            token_queue->pop();
        }
        break;
    default:
      break;
    }
  }


private:

  std::string name;
  TOKEN_T *real_tokens;
  int *real_values;
  T * stored_values;
  std::queue<MY_QUIN*> **trans_queue;
  std::queue<TOKEN_T*> **token_pool;
  std::queue<TOKEN_T*> *token_queue;
  int count;
};



template <class S, class T, int depth, int size>
class ns_master_if_manager_template {//

public:

  typedef class _token_<ns_master_if_manager_template<S, T, depth,size> > TOKEN_T;

  ns_master_if_manager_template(const std::string& name) : name(name) {
     
      real_tokens = new TOKEN_T[depth*size](this);
      t_index = new int[depth*size](this);
      
      tokens = new std::queue<TOKEN_T *> *[size]; 
      token_pool = new std::queue<TOKEN_T *>;
      for (int j = 0; j < size; j++){
        tokens[j] = new std::queue<TOKEN_T *>;
      }
      for (int i=0;i<depth*size;i++) 
        token_pool->push(real_tokens+i);
      }

  ~ns_master_if_manager_template() {
    delete t_index;
    delete [] tokens;
    delete token_pool;
    delete [] real_tokens;
  }

  bool token_free_for_alloc(const S& ind, _base_machine_ *obj) {
   
    if (tokens[ind.val()]->size()>=depth || port_counter_write[ind.val()] == 1){
      return false;
    }else{
      return true;
    }
  }


  bool token_free_for_inquire(const S& ind,_base_machine_ *obj) {
    return true;
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    /* if the token to be reclaimed is the same as the 
     * token at the front of the buffer 
     */
    unsigned offset;
    int my_index;
    offset = unsigned (token - real_tokens);
    my_index = t_index[offset];       
     if (port_counter_read[my_index] == 0){
    if (token == tokens[my_index]->front()){ 
      return true;
   }else{
     return false; 
   }
     }else{
       return false;
     }
  }

  TOKEN_T *allocate_token( const S& ind, _base_machine_ *obj) {//
    int offset;
    TOKEN_T * t;
    
    t = token_pool->front();
    offset = t - real_tokens;
    t_index[offset] = ind.val();
    tokens[ind.val()]->push(t);
    port_counter_write[ind.val()] = 1;
    token_pool->pop();
    t->set_used(obj);
    return t;
 
  }

  void inquire_token(const S& ind,_base_machine_ *obj) {
   
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
    unsigned offset, my_index;
    offset = unsigned (token - real_tokens);
    my_index= t_index[offset];
    tokens[my_index] -> pop();
    token_pool->push(token);      
    port_counter_read[my_index] = 1;
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    reclaim_token(token,obj);    
  
  }  
  
  const std::string& get_name() {return name;}

  void update_on_clock(){
   for (int j = 0; j < size; j++){
    port_counter_write[j] = 0;
    port_counter_read[j] = 0;
   }   
 }
   
private:

  std::string name;
  TOKEN_T *real_tokens;
  int * t_index;
  std::queue<TOKEN_T*> **tokens;
  std::queue<TOKEN_T *> *token_pool;
  int port_counter_read [size];
  int port_counter_write [size];
  
};

template <class S, class T, int arbiter_size>
class ns_bus_arbiter_manager_template {//

public:

  typedef class _token_<ns_bus_arbiter_manager_template<S, T, arbiter_size> > TOKEN_T;

  ns_bus_arbiter_manager_template(const std::string& name) : name(name) {
    
      tokens = new TOKEN_T[arbiter_size](this);
      
      for (int i=0;i<arbiter_size;i++) 
        tokens[i].set_unused();
      for (int i =0; i < arbiter_size; i++){
        request_port[i] = -1;
        grant_port[i] = -1;
        trans_type[i] = 0;        
      }
      cursor_last_ = 0;  
      bus_used_counter = 0;
      bus_busy = false;
      bus_utility = bus_total_count = 0;
  }

  ~ns_bus_arbiter_manager_template() {
   
    delete [] tokens;
  }

  bool token_free_for_alloc(const S& ind,_base_machine_ *obj) {
    return (request_port[(ind.first).val()]==-1);;
  }

  bool token_free_for_inquire(const S& ind,_base_machine_ *obj) {
    return true;
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    /* if the token to the one used for the specific input port,
     * indicated by the grant list, then can proceed
     */
    unsigned offset, port_num;
    offset =  unsigned (token - tokens);
    port_num= offset % arbiter_size;
    if (grant_port[port_num] > -1){
      return true;
    }else{
      return false;
    }
  }

  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    assert((ind.first).val() < arbiter_size);
    tokens[(ind.first).val()].set_used(obj);
    assert(grant_port[(ind.first).val()] ==-1);
    assert(request_port[(ind.first).val()] ==-1);
    request_port[(ind.first).val()] = 1;
    trans_type[(ind.first).val()] = (ind.second).val() ;
    return &(tokens[(ind.first).val()]);
 
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
    //free up the bus counter
     bus_used_counter = 0;
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
   unsigned offset, port_num;
   offset =  unsigned (token - tokens);
   port_num = offset % arbiter_size;
   assert(grant_port[port_num] > -1); 
   grant_port[port_num] = -1;
   request_port[port_num] = -1;
   //bus_busy = false;
   if ((token->get_tenant()==obj))
		token->set_unused();
    
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    reclaim_token(token,obj);    
  
  }  
  
  const std::string& get_name() {return name;}

  void update_on_clock(){
   
    /* need to make a round-robin grant sequence */
    /* init the port_list, here the index is input port number*/
   /*
     for (int j =0; j< arbiter_size; j++){
       grant_port[j] = -1;
  }
   */
     if (bus_used_counter>0){
       /* still in the process of transmitting */
       bus_used_counter --;
       
     }else{
       /* bus vacant */
       
     int i = cursor_last_;
     for (int j=0;j<arbiter_size;j++){
       if (request_port[i]>-1) {
         grant_port[i] = 1;
         bus_busy = true;
         
         //request_port[i] = -1;
         assert(trans_type[i]>=1);
         //bus_used_counter = trans_type[i];
         bus_used_counter = 40;
         bus_used_counter --;
         cursor_last_= (cursor_last_+1)%arbiter_size;
         break;
       }
      i=(i+1)%arbiter_size;
     }
    //if(bus_used_counter ==0) bus_utility ++;
    
    
 }
 #if 0
   bus_total_count ++;
   if (bus_total_count  >= TOTAL_COUNT &&
   bus_total_count % TOTAL_COUNT ==0 ){
        float rate = ((float)bus_utility)/((float)bus_total_count);
        fprintf(stderr, "arb bus utility rate = %f for %d cycles\n", rate, bus_total_count);
   }
 #endif
   assert (bus_used_counter>=0);    
}
private:

  std::string name;
  TOKEN_T *tokens;
  int cursor_last_;
  bool bus_busy;
  int request_port[arbiter_size];
  int grant_port[arbiter_size];
  int trans_type[arbiter_size];
  int bus_used_counter;  
  unsigned bus_total_count, bus_utility;
};

/* this is an implementation of the IBM CoreConnect PLB Bus Arbiter
 * which can support PLB Address Pipeling. 
 * The timing relationship conforms to the the timing diagram listed in 
 * IBM 32-bit Processor Local Bus Architecture Specifications V. 2.9
 * Serial Number: SA-14-2531-01 
 */

template <class S, class T, int arbiter_size, int config>
class ns_cc_bus_arbiter_manager_template {//

public:

  typedef class _token_<ns_cc_bus_arbiter_manager_template<S, T, arbiter_size, config> > TOKEN_T;

  ns_cc_bus_arbiter_manager_template(const std::string& name) : name(name) {
    
      tokens = new TOKEN_T[arbiter_size](this);
      
      for (int i=0;i<arbiter_size;i++) 
        tokens[i].set_unused();
      for (int i =0; i < arbiter_size; i++){
        request_port[i] = -1;
        grant_port[i] = -1;
        trans_type[i] = -1;        
        is_lock[i] = -1;
      }
      cursor_last_ = 0;  
      read_data_bus_used_counter = 0;
      write_data_bus_used_counter = 0;
      bus_utility = bus_total_count = 0;
      write_data_bus_used_counter_secondary_next = BUS_IDLE ;
      write_data_bus_used_counter_secondary = BUS_IDLE;
      read_data_bus_used_counter_secondary_next = BUS_IDLE ;
      read_data_bus_used_counter_secondary = BUS_IDLE;
      delayed_ack_read = false;
      delayed_ack_write = false;
      curr_addr_bus_lock = -1;
      bus_locked = false;
      same_master_slave = false;
  }

  ~ns_cc_bus_arbiter_manager_template() {
   
    delete [] tokens;
  }

  bool token_free_for_alloc(const S& ind,_base_machine_ *obj) {
    return (request_port[(ind.third).val()]==-1 && grant_port[(ind.third).val()]==-1);
  }

  bool token_free_for_inquire(const S& ind,_base_machine_ *obj) {
    //fprintf(stderr,"ind = %d\n", (ind.third).val());
    //assert((ind.third).val()>-1);
    if ( (ind.third).val() == BUS_PRIMARY){
      if (grant_port[(ind.first).val()] == BUS_PRIMARY){
        return true;
      }else{
        //cerr << "arbiter " << ind.first.val() << ". " << ind.third.val() << " not good" <<endl;
        return false;
      }
    }else if ((ind.third).val() == BUS_SECONDARY){
      if (grant_port[(ind.first).val()] == BUS_SECONDARY){
        return true;
      }else{
        return false;
      }
    }else if ((ind.third).val() == BUS_SAME_MASTER_SLAVE){
      return same_master_slave;
     
    }else{
      return true;
    } 
  
  }

  bool token_good_to_reclaim(TOKEN_T *token, _base_machine_ *obj) {
    /* if the token to the one used for the specific input port,
     * indicated by the grant list, then can proceed
     */
    unsigned offset, port_num;
    offset =  unsigned (token - tokens);
    port_num= offset % arbiter_size;
    if (grant_port[port_num] > -1){
      return true;
    }else{
      //cerr << "not good" <<endl;
      return false;
    }
  }

  TOKEN_T *allocate_token(const S& ind, _base_machine_ *obj) {
    assert((ind.third).val() < arbiter_size);
    tokens[(ind.third).val()].set_used(obj);
    assert(grant_port[(ind.third).val()] ==-1);
    assert(request_port[(ind.third).val()] ==-1);   
    is_lock[(ind.third).val()] = (ind.fourth).val();
    /* if it is a pure unlock event */
    if (is_lock[(ind.third).val()] == BUS_UNLOCK && (ind.first).val()== (ind.third).val()){
      bus_locked = false;
      curr_addr_bus_lock =-1;
      fprintf(stderr, "arbiter: separate bus unlocked by %d\n", (ind.first).val());
     
        is_lock[ind.third.val()] = BUS_NO_LOCK;
     
      return &(tokens[(ind.third).val()]);      
    }
    request_port[(ind.third).val()] = 1;
    trans_type[(ind.third).val()] = (ind.second).val();
    return &(tokens[(ind.third).val()]);
 
  }

  void inquire_token(const S& ind, _base_machine_ *obj) {
    switch ((ind.third).val()){
    case BUS_PRIMARY_REL_READ:
       read_data_bus_used_counter = BUS_IDLE;
      break;
    case BUS_SECONDARY_REL_READ:
       if ( current_read_data_bus_addr_secondary == ind.first.val()){
        read_data_bus_used_counter_secondary = BUS_IDLE; 
      }else if ( current_read_data_bus_addr_secondary_next == ind.first.val()) {
        read_data_bus_used_counter_secondary_next = BUS_IDLE;
      }else{
        fprintf(stderr, "arbiter: releasing, don't know which index \n");
      }
      break;
    case BUS_DACK_READ:
      delayed_ack_read = true;
      ack_addr_read = ind.first.val();
      fprintf(stderr, "arbiter: dack read.\n");
      break;
    case BUS_PRIMARY_REL_WRITE:
       write_data_bus_used_counter = BUS_IDLE;
      break;
    case BUS_SECONDARY_REL_WRITE:
       if ( current_write_data_bus_addr_secondary == ind.first.val()){
        write_data_bus_used_counter_secondary = BUS_IDLE; 
      }else if ( current_write_data_bus_addr_secondary_next == ind.first.val()) {
        write_data_bus_used_counter_secondary_next = BUS_IDLE;
      }else{
        fprintf(stderr, "arbiter: releasing, don't know which index \n");
      }
      break;
    case BUS_DACK_WRITE:
      delayed_ack_write = true;
      ack_addr_write = ind.first.val();
      break;  
    case BUS_DELOCK:
      bus_locked = false;
      curr_addr_bus_lock =-1;
      fprintf(stderr, "bus delocked by %d\n", ind.first.val()); 
      break;
    case BUS_RELOCK:
      bus_locked = true;
      curr_addr_bus_lock = ind.first.val();
      fprintf(stderr, "bus relocked by %d\n", ind.first.val());
      break;
    case BUS_SAME_MASTER_SLAVE:
      same_master_slave = false;
      break;     
    default:
      //fprintf(stderr, "unknown inqure of %d\n",ind.third.val()); 
      break;
    }
  }

  void reclaim_token(TOKEN_T *token, _base_machine_ *obj) {
   unsigned offset, port_num;
   offset =  unsigned (token - tokens);
   port_num = offset % arbiter_size;
   assert(grant_port[port_num] > -1); 
   grant_port[port_num] = -1;
   request_port[port_num] = -1;
   if ((token->get_tenant()==obj))
		token->set_unused();
   }

  /** Access the value of the token.
   * This corresponds to val = *token in OSM expression.
   * @param obj The OSM reading the value.
   */
  _UINT_T(8) read_token(_base_machine_ *obj, TOKEN_T *token) {
                 
    if (bus_locked){
      return BUS_LOCKED;
    }else{
      return 0;
    }
  }

  void discard_token(TOKEN_T *token, _base_machine_ *obj) {
    
   if ((token->get_tenant()==obj))
		token->set_unused();  
  }  
  
  const std::string& get_name() {return name;}

  void update_on_clock(){//update
   
    /* need to make a round-robin grant sequence */
    /* init the port_list, here the index is input port number*/
    //handling the read part
    if (read_data_bus_used_counter_secondary_next != BUS_IDLE && read_data_bus_used_counter_secondary == BUS_IDLE){
      read_data_bus_used_counter_secondary = read_data_bus_used_counter_secondary_next;
      read_data_bus_used_counter_secondary_next = BUS_IDLE;
      current_read_data_bus_addr_secondary = current_read_data_bus_addr_secondary_next;
      current_read_data_bus_addr_secondary_next = -1; 
    }
    
    if (delayed_ack_read && 
        read_data_bus_used_counter_secondary != BUS_IDLE && 
        read_data_bus_used_counter == BUS_IDLE && 
        current_read_data_bus_addr_secondary == ack_addr_read)
        {
      // promote the secondary transaction to primary transaction 
          read_data_bus_used_counter = read_data_bus_used_counter_secondary;
          read_data_bus_used_counter_secondary = BUS_IDLE;
          current_read_data_bus_addr = current_read_data_bus_addr_secondary;
          current_read_data_bus_addr_secondary = -1;         
          delayed_ack_read = false;        
        }

    if (read_data_bus_used_counter_secondary_next != BUS_IDLE && read_data_bus_used_counter_secondary == BUS_IDLE){
      read_data_bus_used_counter_secondary = read_data_bus_used_counter_secondary_next;
      read_data_bus_used_counter_secondary_next = BUS_IDLE;
      current_read_data_bus_addr_secondary = current_read_data_bus_addr_secondary_next;
      current_read_data_bus_addr_secondary_next = -1; 
    }
    //handling the write part

    if (write_data_bus_used_counter_secondary_next != BUS_IDLE && write_data_bus_used_counter_secondary == BUS_IDLE){
      write_data_bus_used_counter_secondary = write_data_bus_used_counter_secondary_next;
      write_data_bus_used_counter_secondary_next = BUS_IDLE;
      current_write_data_bus_addr_secondary = current_write_data_bus_addr_secondary_next;
      current_write_data_bus_addr_secondary_next = -1; 
    }
    
    if (delayed_ack_write && 
        write_data_bus_used_counter_secondary != BUS_IDLE && 
        write_data_bus_used_counter == BUS_IDLE && 
        current_write_data_bus_addr_secondary == ack_addr_write)
        {
      // promote the secondary transaction to primary transaction 
          write_data_bus_used_counter = write_data_bus_used_counter_secondary;
          write_data_bus_used_counter_secondary = BUS_IDLE;
          current_write_data_bus_addr = current_write_data_bus_addr_secondary;
          current_write_data_bus_addr_secondary = -1;         
          delayed_ack_write = false;        
        }

    if (write_data_bus_used_counter_secondary_next != BUS_IDLE && write_data_bus_used_counter_secondary == BUS_IDLE){
      write_data_bus_used_counter_secondary = write_data_bus_used_counter_secondary_next;
      write_data_bus_used_counter_secondary_next = BUS_IDLE;
      current_write_data_bus_addr_secondary = current_write_data_bus_addr_secondary_next;
      current_write_data_bus_addr_secondary_next = -1; 
    }


     int i = cursor_last_;
     for (int j=0;j<arbiter_size;j++){
       if (request_port[i]>-1) {
         switch (trans_type[i]){
         case BUS_READ:
           if (read_data_bus_used_counter==BUS_IDLE && read_data_bus_used_counter_secondary == BUS_IDLE &&  read_data_bus_used_counter_secondary_next == BUS_IDLE){   
             if ((is_lock[i] ==BUS_LOCK && curr_addr_bus_lock ==i && bus_locked) ||(is_lock[i] ==BUS_LOCK && curr_addr_bus_lock ==-1 && !bus_locked) ||(!bus_locked && is_lock[i] ==BUS_NO_LOCK) || (curr_addr_bus_lock == i && bus_locked)){
             grant_port[i] = BUS_PRIMARY;
             request_port[i] = -1;             
             read_data_bus_used_counter = BUS_TIMEOUT; //timeout value
             read_data_bus_used_counter --;
             current_read_data_bus_addr = i;
             // see if current bus is requested to be locked
             // the condition is that both read and write bus are idle
             if (is_lock[i] == BUS_LOCK && curr_addr_bus_lock ==-1 && !bus_locked && write_data_bus_used_counter==BUS_IDLE && write_data_bus_used_counter_secondary == BUS_IDLE &&  write_data_bus_used_counter_secondary_next == BUS_IDLE){

               bus_locked = true;
               curr_addr_bus_lock = i;
               fprintf(stderr, "arbiter: bus locked by %d\n", i);
             }

             if (is_lock[i] == BUS_UNLOCK){
               bus_locked = false;
               curr_addr_bus_lock =-1;
               fprintf(stderr, "arbiter: bus unlocked by %d\n", i);
             }

             cursor_last_= (cursor_last_+1)%arbiter_size;
            }
           }else if (read_data_bus_used_counter!=BUS_IDLE && read_data_bus_used_counter_secondary == BUS_IDLE && read_data_bus_used_counter_secondary_next == BUS_IDLE){
             if ((is_lock[i] ==BUS_LOCK && curr_addr_bus_lock ==i && bus_locked) || (!bus_locked && is_lock[i] ==BUS_NO_LOCK) || (curr_addr_bus_lock == i && bus_locked)){
             grant_port[i] = BUS_SECONDARY;
             request_port[i] = -1;             
             read_data_bus_used_counter_secondary = BUS_TIMEOUT; //timeout value
             read_data_bus_used_counter_secondary --;
             current_read_data_bus_addr_secondary = i;
             cursor_last_= (cursor_last_+1)%arbiter_size;
             if (current_read_data_bus_addr_secondary == current_read_data_bus_addr)
               same_master_slave = true;
              if (is_lock[i] == BUS_UNLOCK){
               bus_locked = false;
               curr_addr_bus_lock =-1;
                fprintf(stderr, "arbiter: bus unlocked by %d\n", i);
              }
             }

           }else if  (read_data_bus_used_counter==BUS_IDLE && read_data_bus_used_counter_secondary != BUS_IDLE && read_data_bus_used_counter_secondary_next == BUS_IDLE && config == CC_BUS_ARBITER_PIPELINE){
            
            // see if we can give them the secondary read bus indicator
             if ((is_lock[i] ==BUS_LOCK&& curr_addr_bus_lock ==i && bus_locked) || (!bus_locked && is_lock[i] ==BUS_NO_LOCK)||(curr_addr_bus_lock == i && bus_locked)){
             grant_port[i] = BUS_SECONDARY;
             request_port[i] = -1;             
             read_data_bus_used_counter_secondary_next = BUS_TIMEOUT; //timeout value
             read_data_bus_used_counter_secondary_next --;
             current_read_data_bus_addr_secondary_next = i;
             cursor_last_= (cursor_last_+1)%arbiter_size;
            
             if (is_lock[i] == BUS_UNLOCK){
               bus_locked = false;
               curr_addr_bus_lock =-1;
                fprintf(stderr, "arbiter: bus unlocked by %d\n", i);
             }
            }
           }

         break;
         case BUS_WRITE:
             if (write_data_bus_used_counter==BUS_IDLE && write_data_bus_used_counter_secondary == BUS_IDLE &&  write_data_bus_used_counter_secondary_next == BUS_IDLE){   
              if ((is_lock[i] ==BUS_LOCK && curr_addr_bus_lock ==i && bus_locked) ||(is_lock[i] ==BUS_LOCK && curr_addr_bus_lock ==-1 && !bus_locked) ||(!bus_locked && is_lock[i] ==BUS_NO_LOCK) || (curr_addr_bus_lock == i && bus_locked)){
             grant_port[i] = BUS_PRIMARY;
             request_port[i] = -1;             
             write_data_bus_used_counter = BUS_TIMEOUT; //timeout value
             write_data_bus_used_counter --;
             current_write_data_bus_addr = i;
             if (is_lock[i] == BUS_LOCK&& curr_addr_bus_lock ==-1 && !bus_locked && read_data_bus_used_counter==BUS_IDLE && read_data_bus_used_counter_secondary == BUS_IDLE &&  read_data_bus_used_counter_secondary_next == BUS_IDLE){

               bus_locked = true;
               curr_addr_bus_lock = i;
               fprintf(stderr, "bus locked by %d in write\n", i);
             }
             if (is_lock[i] == BUS_UNLOCK){
               bus_locked = false;
               curr_addr_bus_lock =-1;
               fprintf(stderr, "bus unlocked by %d\n", i);
             }

              cursor_last_= (cursor_last_+1)%arbiter_size;
             }
           }else if (write_data_bus_used_counter!=BUS_IDLE && write_data_bus_used_counter_secondary == BUS_IDLE &&  write_data_bus_used_counter_secondary_next == BUS_IDLE){
              if ((is_lock[i] ==BUS_LOCK && curr_addr_bus_lock ==i && bus_locked) || (!bus_locked && is_lock[i] ==BUS_NO_LOCK)||(curr_addr_bus_lock == i && bus_locked)){
             grant_port[i] = BUS_SECONDARY;
             request_port[i] = -1;             
             write_data_bus_used_counter_secondary = BUS_TIMEOUT; //timeout value
             write_data_bus_used_counter_secondary --;
             current_write_data_bus_addr_secondary = i;
             cursor_last_= (cursor_last_+1)%arbiter_size;
             if (is_lock[i] == BUS_UNLOCK){
               bus_locked = false;
               curr_addr_bus_lock =-1;
                fprintf(stderr, "arbiter: bus unlocked by %d\n", i);
             }
            }
           }else if  (write_data_bus_used_counter==BUS_IDLE && write_data_bus_used_counter_secondary != BUS_IDLE && write_data_bus_used_counter_secondary_next == BUS_IDLE && config == CC_BUS_ARBITER_PIPELINE){
             // see if we can give them the secondary read bus indicator
              if ((is_lock[i] == BUS_LOCK&& curr_addr_bus_lock ==i && bus_locked) || (!bus_locked && is_lock[i] ==BUS_NO_LOCK)||(curr_addr_bus_lock == i && bus_locked)){ 
             grant_port[i] = BUS_SECONDARY;
             request_port[i] = -1;             
             write_data_bus_used_counter_secondary_next = BUS_TIMEOUT; //timeout value
             write_data_bus_used_counter_secondary_next --;
             current_write_data_bus_addr_secondary_next = i;
             cursor_last_= (cursor_last_+1)%arbiter_size;
             if (is_lock[i] == BUS_UNLOCK){
               bus_locked = false;
               curr_addr_bus_lock =-1;
               fprintf(stderr, "arbiter: bus unlocked by %d\n", i);
             }
            }
           }
           break;
         default:
           fprintf(stderr, "ns_cc_bus_arbiter: trans type not known \n");         
         }
       }
      i=(i+1)%arbiter_size;
     }
  }  


private:

  std::string name;
  TOKEN_T *tokens;
  int cursor_last_;
  bool bus_busy;
  bool delayed_ack_read;
  bool delayed_ack_write;
  bool bus_locked;
  bool same_master_slave;
  int curr_addr_bus_lock;
  int ack_addr_read;
  int ack_addr_write;
  int request_port[arbiter_size];
  int grant_port[arbiter_size];
  int trans_type[arbiter_size];
  int is_lock[arbiter_size];
  int read_data_bus_used_counter;
  int write_data_bus_used_counter;
  int read_data_bus_used_counter_secondary;
  int read_data_bus_used_counter_secondary_next;
  int write_data_bus_used_counter_secondary;
  int write_data_bus_used_counter_secondary_next;
  int current_read_data_bus_addr;
  int current_write_data_bus_addr;
  int current_read_data_bus_addr_secondary;
  int current_read_data_bus_addr_secondary_next;
  int current_write_data_bus_addr_secondary; 
  int current_write_data_bus_addr_secondary_next;   
  unsigned bus_total_count, bus_utility;
};


typedef ns_serialize_manager<_UINT_T(8), 1000> my_serialize;

typedef ns_prob_resource_manager<_UINT_T(8)> my_prob_resource;

typedef ns_timer_manager<_UINT_T(8)> my_timer;

typedef ns_reduced_timer_manager<_UINT_T(8)> my_reduced_timer;

typedef ns_phase_resource_manager<int> my_phase_resource;

typedef ns_phase_risingedge_manager<int, 0> my_phase_risingedge;

typedef ns_phase_risingedge_manager<int, 1> my_phase_transedge;

typedef ns_pipe_fetch_manager<_UINT_T(8),_UINT_T(8), 1, PROBABILITY> my_ns_new_fetch_manager;

typedef ns_arm_fetch_manager<_UINT_T(8),_TUPLE_T(_UINT_T(8),_UINT_T(32)), NUM_INSTS> my_ns_arm_fetch_manager;

typedef ns_xbar_manager<_TUPLE_T(_UINT_T(8),_UINT_T(8)), 1, NUM_XBAR_PORTS> my_ns_xbar_manager;


typedef ns_vc_arbiter_manager<_QUINTUPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8),_UINT_T(8), _UINT_T(8)), _UINT_T(8), NETWORK_SIZE, NUM_XBAR_PORTS, VC_SIZE> my_ns_vc_arbiter_manager;

typedef ns_vc_buffer_manager_template<_QUINTUPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8),_UINT_T(8),_UINT_T(8)),_UINT_T(8), NETWORK_SIZE, BUFFER_SIZE, BUFFER_DEPTH, VC_SIZE> ns_vc_buffer_manager;

typedef ns_bus_fetch_manager<_UINT_T(8),_UINT_T(8), 1, BUS_PROB> my_ns_bus_source_manager;

typedef ns_cc_bus_slave_response_manager<_UINT_T(8), _QUINTUPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8),_UINT_T(8), _UINT_T(8)), BUS_NUM_INSTS, GENERIC_READ> my_cc_bus_slave_read_manager;

typedef ns_cc_bus_slave_response_manager<_UINT_T(8), _QUINTUPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8),_UINT_T(8), _UINT_T(8)), BUS_NUM_INSTS, GENERIC_WRITE> my_cc_bus_slave_write_manager;

typedef ns_cc_bus_fetch_manager<_UINT_T(8), _QUADRUPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8),_UINT_T(8)), BUS_NUM_INSTS, BUS_PROB, BUS_CONFIG> my_cc_bus_source_manager;

typedef ns_master_if_manager_template<_UINT_T(8),_UINT_T(8), BUS_BUFFER_DEPTH, BUS_NUM_INSTS> ns_bus_master_if_manager;
typedef ns_master_if_manager_template<_UINT_T(8),_UINT_T(8), BUS_BUFFER_DEPTH, BUS_NUM_INSTS> ns_bus_slave_if_manager;

typedef ns_bus_arbiter_manager_template<_TRIPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8)), _UINT_T(8), BUS_ARBITER_SIZE> my_ns_bus_arbiter_manager;

typedef ns_cc_bus_arbiter_manager_template<_QUADRUPLE_T(_UINT_T(8),_UINT_T(8),_UINT_T(8), _UINT_T(8)), _UINT_T(8), BUS_ARBITER_SIZE, CC_BUS_ARBITER_PIPELINE> my_cc_bus_arbiter_manager;

#endif



