
#include "ns_interface.hpp"

#include "ns_opers.hpp"
#include "ns_machines.hpp"
#include <vector>
#include <iomanip>
#include <iostream>

#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include "config.h"

extern void myrand_init( unsigned int );     /* initialize the generator */
// extern void calc_next_src_tlb();
extern void stat_init();

using std::vector;



#define _MACH_DEF(_a,_b) #_b,
static char *mach_names[] = {
	"dummy",
#include "ns_mach_list.hpp"
};

#define _OPER_DEF(_a,_b, _c) #_b,
static char *oper_names[] = {
	"dummy",
#include "ns_oper_list.hpp"
};

_simulator::_simulator(bool verbose) :
	running(false), verbose(verbose)
{

    /* Create the managers */
#define MANG_DEF(cname, iname) iname = new cname(#iname);
#include "ns_mang_list.hpp"

    /* set up the address */
    int address_c = 0;
#ifdef __NS_ARM_SIM
#define NS_MANG_DEF(cname,iname) iname->set_address(address_c++);
#include "ns_fetch_mang_list.hpp"
#endif


	/* storage for operation/machine */
	machine_pool = new obj_pool<_base_machine_, _MACH_ID_T, _simulator,
								_simulator::mach_allocator>(_mach_total);
	operation_pool = new obj_pool<_base_operation_, _OPER_ID_T, _simulator,
								_simulator::oper_allocator>(_oper_total);
	

	/* reset every thing */
	reset();
}

_simulator::~_simulator()
{
	delete machine_pool;
	delete operation_pool;

#define MANG_DEF(cname, iname) delete iname;
#include "ns_mang_list.hpp"

}

void _simulator::dump_stats()
{
  float f= retire_count/cycle_count;
	fprintf(stderr, "%lld %lld %lld %f\n", alloc_count, retire_count, max_osm_count, f);
}

void _simulator::reset()
{
	cycle_count = 0;
	alloc_count = 0;
	retire_count = 0;
        max_osm_count = 0;
}


void _simulator::clock_tick()
{
	if (running) {

//#define DEBUG
#ifdef DEBUG
		fprintf(stderr, "--clock %d--\n", cycle_count);
#endif
                
                if (work_list.size()> max_osm_count){
                  max_osm_count = work_list.size();
                }

		/** Activate all OSMs in order. */
		for (work_it=work_list.begin(); work_it!=work_list.end();) {


			ins_it = work_it;
			ins_it++;
#ifdef DEBUG
			int oldstate = (*work_it)->__get_state();
#endif
			bool ret = (*work_it)->__activate();
#ifdef DEBUG
			std::cerr << (*work_it)->__get_dynid() << ": ";
			std::cerr << mach_names[(*work_it)->__get_id()]
				   	<< " " << oldstate << "->";
			std::cerr << (*work_it)->__get_state() << " " << std::endl;
#endif

			/* Test if the OSM is back to the initial state. */
			if (ret) { /* no */
				work_it++;
			}
			else { /* yes */
				/* send retired ones to the sink. */
				sink_machine(*work_it, false);
				work_it = work_list.erase(work_it);
			}
		}
		
	
#define MANG_DEF(cname, iname) iname->update_on_clock();
#include "ns_mang_list.hpp"

		cycle_count++;
	}
}

void _simulator::run(uint64_t maxCycle)
{
	initialize();

	running = true;
	while(running && cycle_count<=maxCycle) {
		clock_tick();
	}
}


void _simulator::activate_operation(_base_operation_ *oper,
	_OPER_ID_T *ids, void *cdg)
{
	_base_machine_ *mach = allocate_machine(oper->__get_mach_id());
	oper->__init(mach, ids, cdg);

#ifdef DEBUG
	std::cerr << '[' << oper_names[oper->__get_id()] << ']' << std::endl;
#endif
	work_list.insert(ins_it, mach);
}

_base_machine_ *_simulator::allocate_machine(_MACH_ID_T id)
{
	_base_machine_ *mach = machine_pool->allocate(this,id);
	mach->__set_dynid(alloc_count++);
	return mach;
}

_base_operation_ *_simulator::allocate_operation(_OPER_ID_T id)
{
	return operation_pool->allocate(this,id);
}

void _simulator::sink_machine(_base_machine_ *mach, bool flush)
{
	_base_operation_ **opers;
	int n_oper = mach->__get_opers(opers);
	for (int i=0; i<n_oper; i++)
		free_operation(opers[i]);

	if (!flush) {
		retire_count++;
		mach->__retire();
#if 0
		fprintf(stderr, "0x%08x : 0x%08x %lld\n",
			 static_cast<normal *>(mach)->pc.val(),
			 static_cast<normal *>(mach)->iw.val(), mach->get_dynid());
		fprintf(stderr, "%lld retire\n", mach->get_dynid());
#endif
	}
	else {
		mach->__flush();
#if 0
		fprintf(stderr, "%lld flush\n", mach->get_dynid());
#endif
	}

	machine_pool->reclaim(mach);
}


void _simulator::initialize()
{
	_base_operation_ *oper;
        _base_machine_ *src;

        #define FETCH_DEF(b,a) {oper =allocate_operation(a);src = allocate_machine(b);oper->__init(src, NULL, NULL);work_list.push_back(src);}

        #include "fetch_list.hpp"
      
        /*
        oper =allocate_operation(_oper_half_fetch0);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch1);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch2);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch3);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch4);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch5);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch6);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

        oper = allocate_operation(_oper_half_fetch7);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch8);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);
        
	oper = allocate_operation(_oper_half_fetch9);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch10);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch11);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch12);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch13);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch14);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);

	oper = allocate_operation(_oper_half_fetch15);
	src = allocate_machine(_mach_half_fetcher);
	oper->__init(src, NULL, NULL);
	work_list.push_back(src);
        */
        myrand_init(1);     /* initialize the generator */
	//calc_next_src_tlb();
        stat_init();

}

_base_machine_ *_simulator::mach_allocator(_simulator *s, _MACH_ID_T id)
{
  switch (id) {
#define _MACH_DEF(_a, _b) \
	case _mach_ ## _b: return new _MACH_CLASS(_b)(s);
#include "ns_mach_list.hpp"
	default:
      assert(0);
  }
}

_base_operation_ *_simulator::oper_allocator(_simulator *s, _OPER_ID_T id)
{
  switch (id) {
#define _OPER_DEF(_a, _b, _c) \
	case _oper_ ## _b: return new _OPER_CLASS(_b)();
#include "ns_oper_list.hpp"
    default:
      assert(0);
  }
}

