
#include "simulator.hpp"

#include "machines.hpp"
#include <vector>
#include <iostream>

#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include <string>
#include <fstream>
#include "config.h"

using std::vector;
using std::cerr;
using std::endl;
using std::ostream;
using std::ofstream;
using std::string;

plx_simulator::plx_simulator(bool verbose) :
	verbose(verbose), running(false), pmem(), dmem()
{
	/* Create the managers */
#define MANG_DEF(cname, iname) iname = new cname(#iname);
#include "mang_list.hpp"

	/* conenct the managers. */
	mIF->connect(mBranch, &pmem);
	mMemCtrl->connect(mMemRead, mMemWrite, &dmem, mMemAddr);
	
	/* reset every thing */
	reset();
}

plx_simulator::~plx_simulator()
{
#define MANG_DEF(cname, iname) delete iname;
#include "mang_list.hpp"

}

void plx_simulator::reset()
{
	pmem.reset();
	dmem.reset();

#define MANG_DEF(cname, iname) iname->reset();
#include "mang_list.hpp"

	reset_stats();
}

void plx_simulator::reset_stats()
{
	inst_count = 0;
}

void plx_simulator::dump_stats(FILE *fp)
{
	fprintf(fp, "Total instructions : %lld\n", inst_count);

	fprintf(stderr, "\n");
	fprintf(stderr, "Total data memory pages allocated : %d\n",
		 dmem.get_page_count());
	fprintf(stderr, "Total inst. memory pages allocated : %d\n",
		 pmem.get_page_count());

	fprintf(stderr, "pc = 0x%08x\n", mIF->get_pc());
}

void plx_simulator::dump_iregs(ostream& os)
{
	for (unsigned i=0; i<16; i++) {
		for (unsigned j=2*i; j<2*i+2; j++) {
			string rval = mRF->get_value(j).hex();
			rval.resize(34,' ');
			os << "r" << j;
		   	if (j<10) os << ' '; 
			os	<< " = " << rval;
		}
		os << endl;
	}
}

void plx_simulator::dump_fregs(ostream& os)
{
	for (unsigned i=0; i<16; i++) {
		for (unsigned j=2*i; j<2*i+2; j++) {
			string rval = mFRF->get_value(j).hex();
			rval.resize(34,' ');
			os << "f" << j;
		   	if (j<10) os << ' '; 
			os	<< " = " << rval;
		}
		os << endl;
	}
}

void plx_simulator::dump_preds(ostream& os)
{
	os << "Predicates: ";
	for (unsigned i=0; i<8; i++) {
		os << mPF->get_value(i).val() << " ";
	}
	os << endl;
}

void plx_simulator::dump_pmem(ostream&os, uint32_t addr, uint32_t size)
{
	pmem.dump(os, addr, size);
}

void plx_simulator::dump_dmem(ostream&os, uint32_t addr, uint32_t size)
{
	dmem.dump(os, addr, size);
}

void plx_simulator::dump_core(ostream& os)
{
	ofstream  ofs;
	ofs.open("core.plx");

	dmem.dump(ofs);
	ofs.close();
}

void plx_simulator::run(UInt64 maxInst)
{

	_MACH_CLASS(fetch_normal) *mach = new _MACH_CLASS(fetch_normal)(this);  
                                                              
	running = true;                                           
	while(running && inst_count<=maxInst) {                   
//      for (int i=0;i<15;i++) fprintf(stderr, "%x ", mRF->get_value(i).val());
//      fprintf(stderr, "\n");                                
		mach->__activate();                                   
		inst_count++;                                         
	}
	delete mach;
}
