
#ifndef __PLXSIMUL_H__
#define __PLXSIMUL_H__

#include <iostream>
#include "emumem.h"
#include "more_managers_iss.hpp"

class plx_simulator {

	enum _machine_id {
		_mach_dummy,
		#define _MACH_DEF(_a,_b) _mach_ ## _b,
		#include "mach_list.hpp"
		_mach_total,
	};

	public:

		/** Constructor.
		 *  @param verbose Verbose mode
		 */
		plx_simulator(bool verbose);

		/** Destructor. */
		~plx_simulator();

		/** simulate for n cycles.
		 *  @param n The maximum number of cycles to simulate (may exit).
		 */
		void run(UInt64 n);

		/** stop running. 
		 *  Called by the signal handler or syscall interpreter.
		 */
		void stop() {running = false;}

		/** See if the simulator is running.
		 */
		bool is_running() const {return running;}

		uint64_t get_cycle_count() const {return inst_count;}

		/** stats output. */
		void dump_stats(FILE *fp);

		/** dump integer registers. */
		void dump_iregs(std::ostream& os);

		/** dump floating-point registers. */
		void dump_fregs(std::ostream& os);

		/** dump predicate registers. */
		void dump_preds(std::ostream& os);

		/** dump prog memory . */
		void dump_pmem(std::ostream& os, uint32_t start_addr, uint32_t length);

		/** dump data memory . */
		void dump_dmem(std::ostream& os, uint32_t start_addr, uint32_t length);

		/** dump all data memory . */
		void dump_core(std::ostream& os);

		/** stats resetting. */
		void reset_stats();

		/** load the program into memory. */
		void load_program(const char *);

		/** reset */
		void reset();

#if 0
		/** Get the value of a register. For debugging use. */
		word_t read_gpr(int ind) const {
			return mRF->get_value(ind).val();
		}

		/** These two hooks for the syscall interpretor. */
		void write_gpr(int ind, word_t val) {
			mRF->set_value(ind, val);
		}
#endif


	private:
		bool verbose;

		/* status flags */
		bool running;

		friend void trap_emul(plx_simulator *,
						const _UINT_T(32)& pc, const _UINT_T(23)& num);

	public:

		uint64_t inst_count;	/* number of instructions. */


	public:

		/* the token managers */
#define MANG_DEF(cname, iname) cname *iname;
#include "mang_list.hpp"

	public:

		/* memory saving the real data value */
		emulator::memory pmem, dmem;

};

#endif
