#include "interface.hpp"
#include <math.h>

void frcpa_single(plx_simulator *sim, _UINT_T(128)& rd, const _UINT_T(128)& rs)
{
	_FLT_T f_rs = _U_TO_F(_UINT_XTR2(rs, 31, 0));
	float v_rs = _VAL(f_rs);
    union {
	    float a;
		_WORD_T b;
    } u;
	u.a = 1.0 / v_rs;
	u.b = u.b & 0xffffc000;
		
	rd =_U2U(32,128, _F_TO_U(_FLT_T(u.a)));
}

void frcpsqrta_single(plx_simulator *sim, _UINT_T(128)& rd, const _UINT_T(128)& rs)
{
    _FLT_T f_rs = _U_TO_F(_UINT_XTR2(rs, 31, 0));
	float v_rs = _VAL(f_rs);
    union {
		float a;
		_WORD_T b;
	} u;
 	u.a = 1.0 / sqrt(v_rs);
    u.b = u.b & 0xffffc000;
	
	rd =_U2U(32,128, _F_TO_U(_FLT_T(u.a)));
}

void flog2a_single(plx_simulator *sim, _UINT_T(128)& rd, const _UINT_T(128)& rs)
{
    _FLT_T f_rs = _U_TO_F(_UINT_XTR2(rs, 31, 0));       
	float v_rs = _VAL(f_rs);            
    union {
		float a;
		_WORD_T b;
	} u;
 	u.a = log(v_rs) / log(2.0);
	u.b = u.b & 0xffffc000;
	
	rd =_U2U(32,128, _F_TO_U(_FLT_T(u.a)));
}

void fexp2a_single(plx_simulator *sim, _UINT_T(128)& rd, const _UINT_T(128)& rs)
{
    _FLT_T f_rs = _U_TO_F(_UINT_XTR2(rs, 31, 0));               
	float v_rs = _VAL(f_rs);                        
    union {
		float a;            
		_WORD_T b;                  
	} u;
	u.a = exp(v_rs * log(2.0));                 
	u.b = u.b & 0xffffc000;
	
	rd =_U2U(32,128, _F_TO_U(_FLT_T(u.a)));
}

#include <iostream>
using std::cerr;
using std::endl;

void fatal_error(plx_simulator *sim, const _UINT_T(32)& iw)
{   
	cerr << "Unknown opcode " << iw.hex() << " reached buffer stage." << endl;
}

void trap_emul(plx_simulator *sim,
				const _UINT_T(32)& pc, const _UINT_T(23)& num)
{
	if (sim->verbose)
		cerr << "Trap #" << num.hex() << " at " << pc.hex() << endl;

	switch (num.val()) {
		case 0: //halt
				sim->running = false;
				if (sim->verbose) cerr << "Halted" << endl;
				break;
		case 1: //dump all registers
				sim->dump_iregs(cerr);
				sim->dump_fregs(cerr);
				sim->dump_preds(cerr);
				break;
		case 2: //dump integer registers
				sim->dump_iregs(cerr);
				break;
		case 3: //dump floating point registers
				sim->dump_fregs(cerr);
				break;
		case 4: //dump predicate registers
				sim->dump_preds(cerr);
				break;
		case 5: //dump predicate registers
				cerr << "PC = " << pc.hex() << endl;
				break;
		case 6: //dump prog memory
				cerr << "Prog Memory:" << endl;
				sim->dump_pmem(cerr, pc.val(), 256);
				break;
		case 7: //dump data memory
				cerr << "Data Memory:" << endl;
				sim->dump_dmem(cerr,
							   	(uint32_t)sim->mRF->get_value(20).val(),
								(uint32_t)sim->mRF->get_value(21).val());
				break;
		case 8: //dump data memory
				cerr << "cycles = " << sim->get_cycle_count() << endl;
				break;
		case 9: //dump stats
				sim->dump_stats(stderr);
				break;
		case 10: //dump core
				sim->dump_core(cerr);
				break;
		default: //unknown
				cerr << "Unknown trap code " << num.hex() << endl;
				
	}
}
