#include "loader.h"

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>

using std::cerr;
using std::endl;

using emulator::MEMORY_PAGE_SIZE;

unsigned int bin2hex(char *val)
{
	unsigned ret = 0;
	int len = strlen(val);
	for (int i=len-1; i>=0; i--) {
		ret |= (val[i]-'0') << (len-1-i);
	}
	return ret;
}

void simulator::load_program(const char *filename)
{
	//code to be filled
	FILE *fp;
	unsigned int lineno = 0;

	char *name_buf = new char[strlen(filename)+10];

	// now load program
	strcpy(name_buf, filename);
	strcat(name_buf, ".wa.bin");//program file
	fp = fopen(name_buf, "rt");
	if (fp==NULL) {
		cerr << "Error openning file " << name_buf << endl;
		exit(1);
	}

	if (verbose) cerr << "Loading " << name_buf << endl;

	char buf[1024];
	while (fgets(buf, 1000, fp)!=NULL) {

		lineno++;

		/* tokenize the line */
		char *ptr = strtok(buf, " \t\n");
		unsigned addr;

		if (ptr==NULL) {
			// empty line do nothing.
			continue;
		}
		// get the decimal address
		if (isdigit(ptr[0])) {
			addr = strtoul(ptr, NULL, 10);
			ptr = strtok(NULL, " \t\n");

			if (ptr==NULL || (ptr[0]!='0' && ptr[0]!='1')) {
				cerr << "Error reading file " << name_buf
				 	<< ", line " << lineno << endl;
				exit(1);
			}

			unsigned val = bin2hex(ptr+1);
			//fprintf(stderr, "%s 0x%x\n", ptr, val);
			pmem.write_word(addr, val);
		}
		else {
			cerr << "Error reading file " << name_buf
				 << ", line " << lineno << endl;
			exit(1);
		}
	}


	mIF->set_pc((unsigned)-4);
	mPF->set_value(0, 1);

	fclose(fp);

	// now load program
	strcpy(name_buf, filename);
	strcat(name_buf, ".wd.bin");//program file
	fp = fopen(name_buf, "rt");
	if (fp==NULL) {
		cerr << "No data file, skipping" << endl;
		delete [] name_buf;
		return;
	}

	if (verbose) cerr << "Loading " << name_buf << endl;

	lineno = 0;
	while (fgets(buf, 1000, fp)!=NULL) {

		lineno++;

		/* tokenize the line */
		char *ptr = strtok(buf, " \t\n");
		unsigned addr;

		if (ptr==NULL) {
			// empty line do nothing.
			continue;
		}
		// get the decimal address
		if (isdigit(ptr[0])) {
			addr = strtoul(ptr, NULL, 10);
			ptr = strtok(NULL, " \t\n");

			if (ptr==NULL || (ptr[0]!='0' && ptr[0]!='1')) {
				cerr << "Error reading file " << name_buf
				 	<< ", line " << lineno << endl;
				exit(1);
			}

			unsigned val = bin2hex(ptr+1);
			//fprintf(stderr, "%s 0x%x\n", ptr, val);
			dmem.write_word(addr, val);
		}
		else {
			cerr << "Error reading file " << name_buf
				 << ", line " << lineno << endl;
			exit(1);
		}
	}

	delete [] name_buf;
}
