
#include <iostream>
#include "more_managers.hpp"
#include "simulator.hpp"

void fetch_manager::update_on_clock()
{
	/* a new OSM comes in */
	if (mRS->branch_taken()) {
		assert(trigger); // a new osm must have come in
		pc = (mRS->get_target());
	}
	else if (trigger) {
		pc += 4;		// increment pc
	}

	if (fetch_status==1) {
		if (itlb->read(pc, 4)) {
			fetch_status = 2;
		}
	}
	if (fetch_status==2) {
		if (icache->read(pc, 4)) {
			fetch_status = 0;
		}
	}
	trigger = false;
}

void mem_ctrl_port::update_on_clock() 
{
	if (trigger) {

		assert(status==0);
		status = 1;
		addr = mMemAddr->get_value().val();
		trigger = false;
	}

	if (status==1) {
		if (dtlb->read(addr, size))
			status = 2;
	}

	if (status==2) {
		if (isRead && dcache->read(addr, size)) {
			if (size==16) {
				_UINT_T(128) result;
				dword_t vlo = mem->read_dword(addr);
				dword_t vhi = mem->read_dword(addr+8);
#if TARGET_LITTLE_ENDIAN //vlo has low order bits
				result = vhi;
				result = (result << 64) | _UINT_CTR(128, vlo);
#else
				result = vlo;
				result = (result << 64) | _UINT_CTR(128, vhi);
#endif
				mMemRead->set_value(result);
			}
			else if (size==8) {
				mMemRead->set_value(mem->read_dword(addr));
			}
			else if (size==4) {
				mMemRead->set_value(mem->read_word(addr));
			}
			else if (size==2) {
				mMemRead->set_value(mem->read_half_word(addr));
			}
			else if (size==1) {
				mMemRead->set_value(mem->read_byte(addr));
			}
			else {
				std::cerr << "unimplemented memory read size "
					  	<<  size << std::endl;
				abort();
			}
			// @@@ more types
			status = 0;
#ifdef _DEBUG_MEM
			std::cerr << "read at " << std::hex << addr << ": "
				   << mMemRead->get_value().hex() << std::endl;
#endif
		}
		if (!isRead && dcache->write(addr, size)) {
			if (size==16) {
				_UINT_T(128) result = mMemWrite->get_value();
				dword_t vlo = (dword_t)result.val();
				dword_t vhi = (dword_t)(result>>64).val();
#if TARGET_LITTLE_ENDIAN //vlo has low order bits
				mem->write_dword(addr, vlo);
				mem->write_dword(addr+8, vhi);
#else
				mem->write_dword(addr, vhi);
				mem->write_dword(addr+8, vlo);
#endif
			}
			else if (size==8) {
				mem->write_dword(addr, (uint64_t)mMemWrite->get_value().val());
			}
			else if (size==4) {
				mem->write_word(addr, (uint32_t)mMemWrite->get_value().val());
			}
			else if (size==2) {
				mem->write_half_word(addr, 
							(uint16_t)(uint32_t)mMemWrite->get_value().val());
			}
			else if (size==1) {
				mem->write_byte(addr,
							(uint8_t)(uint32_t)mMemWrite->get_value().val());
			}
			else {
				std::cerr << "unimplemented memory write size "
					   	<< size << std::endl;
				abort();
			}

			// @@@ more types
			status = 0;
#ifdef _DEBUG_MEM
			std::cerr << "write at " << std::hex << addr << ": "
				   << mMemWrite->get_value().hex() << std::endl;
#endif
		}
	}
}
