
#include <cstdio>
#include <csignal>
#include <cstdlib>
#include "simulator.hpp"

#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#endif


static void usage(char *fname)
{
	char *cp;
	if ((cp = rindex(fname, '/'))!=NULL) cp++;
	else cp = fname;
	fprintf(stderr, 
		"usage : %s [-m maxcycle] [-h]\n"
		"  -m : maximum number of cycles to simulate\n"
		"  -h : print this message and quit\n", cp);
}

simple_simulator *sima;

uint64_t ato_uint64(char *input)
{
	char *p = input;
	uint64_t val = 0;
	while (*p!=0) {
		val = val*10+(*p-'0');
		p++;
	}
	return val;
}

void sigint_handler(int signum)
{
	if (sima->is_running()) {
		sima->stop();
	}
	else exit(0);
}

int main(int argc, char *argv[], char *envp[])
{
	int i;
	bool verbose = false;
	uint64_t max_cnum = (uint64_t)-1;
	void (*prev_sigint_handler)(int);

	for(i = 1; i < argc; i++)
	{
		if(strcmp(argv[i], "-m") == 0) max_cnum = ato_uint64(argv[++i]); else
		if(strcmp(argv[i], "-h") == 0) {usage(argv[0]); return 0;} else
		{
			usage(argv[0]);
			exit(1);
		}

	}
	
		sima = new simple_simulator(verbose);
		prev_sigint_handler = signal(SIGINT, sigint_handler); 

#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H
		struct timeval begin_u, end_u, begin_s, end_s;
		struct rusage usg;
		float user_time, sys_time;
		getrusage(RUSAGE_SELF, &usg);
		begin_u = usg.ru_utime;
		begin_s = usg.ru_stime;
#endif

		sima->run(max_cnum);
		if(prev_sigint_handler != SIG_ERR) signal(SIGINT, SIG_DFL);

		sima->dump_stats(stderr);

#if HAVE_SYS_RESOURCE_H && HAVE_SYS_TIME_H
		getrusage(RUSAGE_SELF, &usg);
		end_u = usg.ru_utime;
		end_s = usg.ru_stime;
		user_time = (end_u.tv_sec+end_u.tv_usec/1000000.0)-
			(begin_u.tv_sec+begin_u.tv_usec/1000000.0);
		sys_time = (end_s.tv_sec+end_s.tv_usec/1000000.0)-
			(begin_s.tv_sec+begin_s.tv_usec/1000000.0);
		fprintf(stderr,
			"Total user time  : %.3f sec.\n"
			"Total system time: %.3f sec.\n"
			"Simulation speed : %.3e cyc./sec.\n",
			user_time, sys_time, sima->get_cycle_count()/(user_time+sys_time));
#endif

	return 0;
}

