
#ifndef __MAD_ANNOT_AST_HPP__
#define __MAD_ANNOT_AST_HPP__

#include "misc.hpp"
#include "mad_ast.hpp"
#include <iostream>

using mad::Symbol;
using mad::SymNULL;

namespace mad_ast {


	/* forward declaration. */
	class annot_paragraph;

	/** Class for annotation. */
	typedef class annotation_t : public ast_node {

		public:

			/** Constructor. */
			annotation_t(char *s, const Location *loc) : 
				ast_node(loc), str(s) {}

			/** Destructor. */
			~annotation_t() {free(str);}

			/** Virtual print out function. */
			std::ostream& print(std::ostream &os) const;

			/** Get the annotation string. */
			const char *get_content() const {return str;}

		private: 

			char *str;

	} Annotation;

	/** Class for annotation owner. */
	typedef class annot_owner {

		public:
			/** Constructor. */
			annot_owner() : annots(NULL),
		   		parsed_annots(NULL), parsed(false) {}

			/** Destructor. */
			~annot_owner();

			/** Append annotations to the end of the array. */
			void append_annotation(annotation_t *aa);

			/** Get the annotations.
			 *  @return The annotation array.
			 */
			const std::vector<annotation_t *> *get_annotations() const {
				return annots;
			}

			/** Get the parsed annotations.
			 *  @return The parsed annotations.
			 */
			const std::vector<annot_paragraph *> *
				get_parsed_annotations() const{
				return parsed_annots;
			}

			/** Check the annotations.
			 *  @return True if Ok.
			 */
			bool check_annotations();

		protected:

			std::vector<annotation_t *> *annots;
			std::vector<annot_paragraph *> *parsed_annots;
			bool parsed;

	} AnnotOwner;
}


#endif
