
#ifndef __MAD_AST_HPP__
#define __MAD_AST_HPP__

#include <iostream>
#include <vector>
#include "misc.hpp"

namespace mad_ast {

	/** Base class of all AST nodes */
	typedef class ast_node {

		public:

			/** Constructor.
			 *  @param loc Location of the node.
			 */
			ast_node (const Location *loc) : loc(loc) {}

			/** Suppress auto-generated members, should not be implemented. */
			ast_node(const ast_node&);

			/** Suppress auto-generated members, should not be implemented. */
			ast_node& operator = (const ast_node&);

			/** Virtual destructor for base class. */
			virtual ~ast_node() {delete loc;}

			/** Purely virtual print out function. */
			virtual std::ostream& print(std::ostream &) const = 0;

			/** Get the location of a node */
			const Location *get_loc() const {return loc;}

			/** Set the location of a node. */
			void set_loc(const Location *l) {loc = l;}

		private:
			const Location *loc;

	} ASTNode;

}

#endif

