
#include <iostream>
#include <vector>
#include "project.hpp"
#include "misc.hpp"

using std::cerr;
using std::endl;
using std::vector;
using namespace mad;
using namespace mad_ast;

static void usage(char *fname)
{
	char *cp;

	if ((cp = strrchr(fname, '/'))!=NULL) cp++;
	else cp = fname;
	cerr << 
		"**** " PACKAGE " Version " VERSION " (C) 2003 Wei Qin ****\n"
		"usage : " << cp << " [-h] [-q] [-s] [-p] [-P prefix] [-O] [-I path]"
		" <file name>\n"
		"  -h : print this message and quit\n"
		"  -q : quiet mode, do not display warnings\n"
		"  -s : syntax checking only\n"
		"  -p : pretty-print the program\n"
		"  -P : prefix the names of output files\n"
		"  -O : optimize the program\n"
		"  -F : emit functional simulator (ISS) with optimization\n"
		"  -I : search path for MADL files\n"
		"  file name : the main mad file\n";
}


int main(int argc, char *argv[])
{
	int i;
	bool quiet = false;		// donot display warnings
	bool syntax = false;	// check syntax only
	bool print = false;		// pretty print the program
	bool annot = false;		// check annotation
	bool optimize = false;	// check annotation
	bool emit_iss = false;	// emit ISS
	vector<const char *> ipaths;
	const char *prefix = "";

	char *filename = NULL;
	for (i=1; i<argc; i++)
	{
		if (strcmp(argv[i], "-h")==0) {
			usage(argv[0]);
		   	return 0;
		}
	   	else if (strcmp(argv[i], "-q")==0) {
			quiet = true;
		}
		else if (strcmp(argv[i], "-a")==0) {
			annot = true;
		}
		else if (strcmp(argv[i], "-s")==0) {
			syntax = true;
		}
		else if (strcmp(argv[i], "-p")==0) {
			print = true;
		}
		else if (strcmp(argv[i], "-P")==0) {
			prefix = argv[++i];
		}
		else if (strcmp(argv[i], "-I")==0) {
			ipaths.push_back(argv[++i]);
		}
		else if (strcmp(argv[i], "-F")==0) {
			optimize = true;
			emit_iss = true;
		}
		else if (strcmp(argv[i], "-O")==0) {
			optimize = true;
		}
		else if (!filename) {
			filename = argv[i];
		}
		else {
			usage(argv[0]);
			return 1;
		}
	}

	if (!filename) {
		usage(argv[0]);
		return 1;
	}

	mad_ast_init(quiet);

	project_manager pm(filename, ipaths, syntax,
				print, annot, optimize, emit_iss, prefix);
	if (pm.get_status()) return 0;

	return 1;
}
