
#ifndef __MAD_PROJECT_HPP__
#define __MAD_PROJECT_HPP__

#include "program_ast.hpp"
#include "misc.hpp"
#include <vector>

namespace mad {

	/** Driver class managing file openning, parsing and processing. */
	class project_manager {

		public:
			/** Constructor.
			 *  @param mfile The name of the main file.
			 *  @param ipath The including paths.
			 *  @param syntax Do syntax checking only.
			 *  @param print  To pretty print the program.
			 *  @param annot  To check annotation syntax.
			 *  @param optimize To optimize the program.
			 *  @param iss    To emit ISS.
			 *  @param prefix To prefix output files.
			 */
			project_manager(const char *mfile,
			   	const std::vector<const char *>& ipath,
				bool syntax, bool print, bool annot, bool optimize,
				bool iss, const char *prefix);

			/** close the project. */
			~project_manager();

			/** Get the status, true means ok. */
			bool get_status() const {return status;}

			mad_ast::Program *get_program() {return prog;}

		private:
			/** process the project. */
			void process();

			/** Open a MADL project.
			 *  @return True if ok, otherwise error.  */
			bool open_project();

			/** Open a MADL file.  */
			std::vector<mad_ast::ProgSection *> *open_file(const char *fname);

			/** The name of the main project file. */
			const char *main_file;

			/** The include paths. */
			const std::vector<const char *>& inc_paths;

			bool syntax_only;
			bool pretty_print;
			bool check_annot;
			bool to_optimize;
			bool to_iss;
		
			mad_ast::Program *prog;
			bool status;
			const char *prefix;
	};
}


#endif
