%{

#include "annotation.hpp"
#include <sstream>

using namespace mad;
using namespace mad_ast;
#define YYLTYPE mad_ast::yyltype_t
#include "parse_annot.hpp"

#define YY_NO_UNPUT
#define YY_USER_INIT current_input_pos=0; yylineno = annot_loc.first_line;

#define YY_INPUT(buf,result,max_size) { \
	int c=annot_to_parse[current_input_pos++]; \
	result=(c==0)?YY_NULL:(buf[0]=c,1); \
	}

#define YY_USER_ACTION  annot_loc.advance_column(YYLeng());

static int current_input_pos;
const char *annot_to_parse;
bool annot_parse_error;
Symbol annot_filename;

extern int annot_parse();
extern YYLTYPE annot_loc;

/* some globals for parsing annotation */
annot_FlexLexer *annot_lexer;	/* Annotation lexer. */
AnnotParagraph *annot_parser_result;

static void annot_lexerror(const mad_ast::yyltype_t *loc, char c)
{
 	std::stringstream sstr;
	sstr << "Unknown character " << c << " at " <<
		symbol_get_string(annot_filename) << ":" << *loc;
	error(sstr.str());
}

%}

%option c++
%option yylineno
%option noyywrap
%pointer

D		[0-9]
L		[a-zA-Z_]
E		[Ee][+-]?{D}+
NUM		[0-9]+
HEXNUM  0[xX][0-9a-fA-F]+

DIG     [0-9]
EXP     [Ee][+-]?{DIG}+

STR     \"(\\.|[^\\"])*\"

WHITES  [ \t]*
IDENT   [a-zA-Z_$%][a-zA-Z_0-9$%]*

%x comment

%%

%{
	/* At each yylex invocation, mark the current position as the
	start of the next token.  */
	(annot_loc.step());
%}


"VAR"	return TK_VAR;
"var"	return TK_VAR;
"DEFINE"	return TK_DEF;
"define"	return TK_DEF;
"string"	return TK_STRING;
"int"	return TK_INT;
"uint"	return TK_UINT;
"<:"	return TK_IN;
"=="	return TK_EQ;	
"("		return '(';
")"		return ')';
"{"		return '{';
"}"		return '}';
"<"		return '<';
">"		return '>';
":"		return ':';
";"		return ';';
","		return ',';
"="		return '=';
"+"		return '+';
"-"		return '-';

{STR}		annot_lval.strval = strdup(YYText()); return TK_STR;
{IDENT}		annot_lval.strval = strdup(YYText()); return TK_IDENT;
{NUM}		annot_lval.strval = strdup(YYText()); return TK_NUM;
{HEXNUM}	annot_lval.strval = strdup(YYText()); return TK_NUM;

{WHITES}	/*ignore*/	
"\n"+		annot_loc.advance_line(YYLeng()); annot_loc.step();
<<EOF>>		return 0;

"##"			BEGIN(comment);
<comment>"\n"+	annot_loc.advance_line(YYLeng()); annot_loc.step();
<comment>.		/*ignore*/
<comment>"##"	BEGIN(INITIAL);

"#"		{int c;
		 while ((c=yyinput()) != 0)
			if (c=='\n') break;
		 annot_loc.advance_line(1); annot_loc.step();
		}

.		{
			annot_lexerror(&annot_loc, *YYText());
			annot_loc.step();
		}

%%


void annot_error (char *s)
{
	stringstream sstr;
	sstr << s << " near " << symbol_get_string(annot_filename)
		 << ":" << annot_lexer->lineno();
	fatal(sstr.str());
	annot_parse_error = true;
}

int annot_lex()
{
    return annot_lexer->yylex();
}

AnnotParagraph *annot_parse_wrapper(const char *annot, const Location *loc)
{
	annot_filename = loc->file_name;
	annot_loc = *(loc->loc);

	annot_parser_result = NULL;
	annot_parse_error = false;
	annot_to_parse = annot;
	annot_lexer = new annot_FlexLexer();
	annot_parse();

//extern int annot_counter;
//std::cerr << annot_counter << " annotation paragraphs" << std::endl;
	delete annot_lexer;
	return annot_parser_result;
}
